#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/directory_handling.rb' 
# =========================================================================== #
module FtpParadise
         
class Connection

  require 'ftp_paradise/connection/remote_pwd.rb'
  require 'ftp_paradise/connection/file_handling.rb'

  # ========================================================================= #
  # === report_current_remote_dir
  #
  # Gives you the current dir.
  # ========================================================================= #
  def report_current_remote_dir(
      be_verbose = be_verbose?
    )
    begin
      if be_verbose
        opnn; e 'The remote directory is:'
        e "  #{sdir(return_remote_pwd)}"
      end
    rescue Exception => error
      opnn; e error.class
      opnn; e error
    end
  end; alias report_remote_directory report_current_remote_dir # === report_remote_directory
       alias dir?                    report_current_remote_dir # === dir?
       alias report_remote_pwd       report_current_remote_dir # === report_remote_pwd

  # ========================================================================= #
  # === remote_chdir                                      (cd tag, chdir tag)
  #
  # This method allows us to cd to another remote directory, on the
  # remote FTP server (our main connection). 
  #
  # The method must be able to check whether we are connected to the
  # remote host.
  # ========================================================================= #
  def remote_chdir(
      i,
      be_verbose = be_verbose?
    )
    i = i.dup if i.frozen?
    # ======================================================================= #
    # Sanitize odd entries ending with ':'
    # ======================================================================= #
    i[-1,1] = '/' if i.end_with? ':'
    i = rds("#{i}/") # <- Always ensure that a '/' is the last character.
    if be_verbose
      notify_the_user_about(i, __method__)
    end
    if is_connected?
      # ===================================================================== #
      # If we cd to a non-existing target, the exception Net::FTPPermError
      # will be raised.
      # ===================================================================== #
      begin
        ftp_object?.chdir(i)
        update_remote_file_listing
      # ===================================================================== #
      # The user may have insufficient permissions:
      # ===================================================================== #
      rescue Net::FTPPermError => error
        pp error
      rescue Net::ReadTimeout => error
        opnn; e 'An error was encountered in the method '\
                'remote_chdir().'
        opnn; e 'The error-class was Net::ReadTimeout.'
        e
        opnn; e 'The specific error will be displayed next.'
        e
        pp error
      end
    else
      opnn; e 'We are not connected to a remote host right now.'
      opnn; e 'Thus we can not change the directory.'
    end
    report_current_remote_dir if be_verbose # right now mandatory
  end; alias rcd                           remote_chdir # === rcd
       alias remote_cd                     remote_chdir # === remote_cd
       alias chdir                         remote_chdir # === chdir
       alias cd                            remote_chdir # === cd
       alias chdir                         remote_chdir # === chdir
       alias rcd                           remote_chdir # === rcd
       alias change_directory              remote_chdir # === change_directory
       alias change_remote_directory       remote_chdir # === change_remote_directory
       alias cd_into_this_remote_directory remote_chdir # === cd_into_this_remote_directory
       alias remote_change_directory       remote_chdir # === remote_change_directory

  # ========================================================================= #
  # === return_remote_directories
  #
  # Give us a list of (remote) directories - only directories, not
  # files.
  #
  # An Array will be returned as a result.
  # ========================================================================= #
  def return_remote_directories
    _ = []
    update_file_listing # Populate the file listing anew, so we will always get proper results back.
    raw_listing?.each {|entry|
      entry = FtpParadise::Entry.new(entry)
      _ << rds(entry.return_name) if entry.is_a_directory?
    }
    # ======================================================================= #
    # Directories must end with '/'. Ensure this to be the case next.
    # ======================================================================= #
    _.map! {|entry|
      entry << '/' unless entry.end_with? '/'
      entry
    }
    return _
  end; alias directories?       return_remote_directories # === directories?
       alias return_directories return_remote_directories # === return_directories

  # ========================================================================= #
  # === return_remote_directory_content
  #
  # This method will return ALL remote entries, no matter if file or
  # directory.
  # ========================================================================= #
  def return_remote_directory_content
    raw_entries = raw_entries?
    if raw_entries.nil?
      populate_raw_entries
      raw_entries = raw_entries?
    end
    if raw_entries
      _ = raw_entries.reject {|line|
        line.empty?
      }.map {|entry|
        FtpParadise::Entry.new(entry).name?
      }
    end
    return _
  end; alias array_file_listing?      return_remote_directory_content # === array_file_listing?
       alias array_file_listing       return_remote_directory_content # === array_file_listing
       alias remote_directory_content return_remote_directory_content # === remote_directory_content
       alias list?                    return_remote_directory_content # === list?

  # ========================================================================= #
  # === is_a_directory?
  #
  # This method queries whether the given input is a (remote) directory.
  # For this to work, the given input must obviously exist as file or
  # directory.
  # ========================================================================= #
  def is_a_directory?(i)
    update_file_listing
    candidates = file_listing_as_entries?.select {|entry|
      entry.is_a_directory?
    }.map(&:last)
    candidates.include? i
  end; alias is_directory?                 is_a_directory? # === is_directory?
       alias is_a_directory?               is_a_directory? # === is_a_directory?
       alias this_remote_directory_exists? is_a_directory? # === this_remote_directory_exists?
       alias is_dir?                       is_a_directory? # === is_dir?

  # ========================================================================= #
  # === remote_directory_is_empty?
  #
  # This method will return a Boolean - true if the remote directory
  # is empty, and false otherwise.
  # ========================================================================= #
  def remote_directory_is_empty?
    _ = return_remote_directory_content
    _.reject! {|entry|
      (entry == '.') or (entry == '..')
    }
    _.empty?
  end

  # ========================================================================= #
  # === create_this_remote_directory                              (mkdir tag)
  #
  # This method will attempt to create a remote directory.
  #
  # A Net::FTPPermError may result if the permission does not allow for
  # that action.
  # ========================================================================= #
  def create_this_remote_directory(
      i, be_verbose = be_verbose?
    )
    if i.is_a? Array
      i.each {|entry| create_this_remote_directory(entry, be_verbose) }
    else
      i = rds(i+'/')
      if be_verbose
        notify_the_user_about(i, __method__)
        if is_directory?(i)
          opnn; e swarn('Warning - the remote directory ')+
                  sdir(i)+swarn(' already exists.')
          opnn; e swarn('We thus can not create a new directory.')
        end
      end
      begin
        ftp_object?.mkdir(i)
      rescue Net::FTPConnectionError
        opnn; e 'Not connected - is the network down/available?'
      rescue Net::FTPPermError
        opnn; e "Net::FTPPermError: Can not create the remote "\
                "directory `#{remote_path?}#{sdir(i)}`"
        opnn; e 'as it - or a file with the same name - already exists.'
      end
    end
  end; alias mkdir                           create_this_remote_directory # === mkdir
       alias rmkdir                          create_this_remote_directory # === rmkdir
       alias remote_mkdir                    create_this_remote_directory # === remote_mkdir
       alias create_directory                create_this_remote_directory # === create_directory
       alias create_these_remote_directories create_this_remote_directory # === create_these_remote_directories
       alias remote_create_directory         create_this_remote_directory # === remote_create_directory
       alias create_remote_directory         create_this_remote_directory # === create_remote_directory

  # ========================================================================= #
  # === sanitize_remote_directory_content
  #
  # The purpose of this method is to clean up the remote dataset.
  #
  # Keep in mind that the output comes in something like:
  #
  #    "-rw-rw-rw-   1 web      netscape     6820 May 30  2011 AppendToCookbook.rb"
  #    "drwxrwxrwx   2 web      netscape     8192 Oct 29  2007 CSS"
  #    "-rw-rw-rw-   1 web      netscape    10548 May 30  2011 Chained.rb" 
  #    "-rw-r--r--   1 330        330      237904 Jun  9 14:24 11.03.2003_UniWien_Biologie_EinfuehrungInDieMikrobiologie.jpg",
  #
  # This should contain all the information that is needed.
  # ========================================================================= #
  def sanitize_remote_directory_content
    _ = []
    raw_entries?.each { |e|
      next if e.empty?
      entry = FtpParadise::Entry.new(e)
      filesize  = entry.filesize?
      timestamp = entry.parsed_timestamp?
      # ===================================================================== #
      # === Exclude 3 entries next
      # ===================================================================== #
      next if e.include? 'total' or
              entry.name? == '.' or
              entry.name? == '..' # Batch next.
      # ===================================================================== #
      # === Build up the final Array
      #
      # Our Array will have four entries:
      #
      #   (1) Filename
      #   (2) File or Directory (String)
      #   (3) Filesize
      #   (4) Timestamp
      #
      # ===================================================================== #
      _ << [ rds(entry.filename?), entry.file_or_directory, filesize, timestamp ]
    }
    @internal_hash[:sanitized_remote_directory_content] = _ #.compact
  end

  # ========================================================================= #
  # === sanitized_remote_directory_content?
  # ========================================================================= #
  def sanitized_remote_directory_content?
    update_remote_file_listing
    sanitize_remote_directory_content
    @internal_hash[:sanitized_remote_directory_content]
  end

end; end