#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/do_login.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/toplevel_methods/remote_url.rb'

  require 'ftp_paradise/connection/initialize_a_new_net_ftp_object_with_this_url.rb'
  require 'ftp_paradise/connection/is_connected.rb'
  require 'ftp_paradise/connection/password.rb'
  require 'ftp_paradise/connection/sync_ftp_object_onto_the_main_namespace.rb'

  # ========================================================================= #
  # === do_login
  #
  # This method is responsible for the login-action.
  #
  # The method signature for the net-ftp class' .login() is:
  #
  #   login(user = "anonymous", passwd = nil, acct = nil)
  #
  # This method here accepts a Hash as first argument.
  #
  # For official information about .login(), see here:
  #
  #   https://ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html#method-i-login
  #
  # ========================================================================= #
  def do_login(
      to_this_url          = nil,         # The remote URL.
      use_this_as_username = username?,   # The login-name.
      use_this_as_password = password?,   # The password for that user.
      be_verbose           = be_verbose?
    )
    if to_this_url.nil?
      to_this_url = FtpParadise.remote_url?
    elsif to_this_url.is_a? Hash
    end
    # ======================================================================= #
    # Rescue nil-passwords here.
    # ======================================================================= #
    if use_this_as_password.to_s.empty?
      use_this_as_password = set_password(:try_to_use_a_default_password)
    end
    if be_verbose
      # ===================================================================== #
      # Display some info to the user here.
      # ===================================================================== #
      e; cliner
      opnn; e "Using the URL #{royalblue(to_this_url)} (remote FTP host)."
      e
      e "  Username: #{slateblue(use_this_as_username)}"
      e "  Password: #{slateblue(use_this_as_password)}"
      e
      report_transfer_mode_in_use
      e
      cliner
    end
    if ftp_object?.nil? and !to_this_url.nil?
      initialize_a_new_net_ftp_object_with_this_url(to_this_url)
    end
    use_this_as_username = use_this_as_username.to_s
    # ======================================================================= #
    # The following code must be rescued because the login() method
    # may fail in various ways.
    # ======================================================================= #
    begin
      ftp_object?.login(
        use_this_as_username,
        use_this_as_password
      )
      # =================================================================== #
      # Next, assign towards the toplevel reference to the ftp-connection.
      # =================================================================== #
      sync_ftp_object_onto_the_main_namespace if ftp_object?
      if be_verbose and is_connected?
        e
        cliner {
          e "- #{sfancy('Successfully')} logged into "\
            "the remote FTP host at `#{simp(to_this_url)}`"
        }
      end
    rescue Net::FTPPermError => error
      opnn; e "We could not connect and login to `#{sfancy(to_this_url)}`."
      opnn; e 'It may be that you lack the proper permissions.'
      opnn; e 'This may happen because the user/password combination was'
      opnn; e 'incorrect. User and password combination will be shown next:'
      show_user_and_password
      opnn; e "Feedbacking that error next, of class #{error.class.to_s}:"
      opnn; e orange(error.to_s)
    rescue Net::FTPConnectionError => error
      opnn; e 'We do not seem to be connected. An error happened.'
      pp error
      pp error.class
    end
  end; alias do_connect do_login # === do_connect
       alias connect_to do_login # === connect_to
       alias connect    do_login # === connect
       alias login      do_login # === login
       alias login_to   do_login # === login_to
       alias reconnect  do_login # === reconnect

end; end