#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/file_handling.rb'
# =========================================================================== #
module FtpParadise

class Connection

  # ========================================================================= #
  # === create_remote_file                                        (touch tag)
  #
  # This "touches" a remote file.
  # ========================================================================= #
  def create_remote_file(
      i,
      be_verbose = be_verbose?
    )
    if i.is_a? Array
      i.each {|entry| create_remote_file(entry, be_verbose) }
    else
      FileUtils.touch(i)
      if be_verbose
        notify_the_user_about(i, __method__)
      end
      upload_this_text_file(i, :be_quiet)
      File.delete(i) if File.zero? i # And get rid of that file too.
    end
  end; alias touch create_remote_file # === touch

  # ========================================================================= #
  # === fancy_remote_listing
  #
  # This method will obtain the (remote) directory listing and then
  # display it.
  # ========================================================================= #
  def fancy_remote_listing(
      be_verbose = be_verbose?
    )
    update_file_listing
    case be_verbose
    when :be_silent,
         :be_quiet
      be_verbose = false
    when :also_show_filesize
      be_verbose = true
    end
    if be_verbose
      e ('=' * 55)+' '+
        FtpParadise.yellow('REMOTE FILES')+' '+('=' * 12)
      if remote_directory_is_empty?
        e "Remote `#{sdir(remote_path?)}` is empty."
        e 'No files or directories can be found therein.'
      else
        e 'Remote content of `'+sdir(remote_path?)+'` is:'+N+N
        e 'Index  Filename                           '\
          '                                    Filesize'
        show_remote_file_listing
      end
      cliner
    end
    return remote_directory_content
  end

  # ========================================================================= #
  # === rename
  #
  # This method will attempt to rename a file on the remote server.
  #
  # Note that this is equivalent to moving a file, which is why the alias
  # move_file can also be used. Any move-file actioni s simply a rename()
  # action.
  #
  # The first argument passed to this method should be the name of an 
  # existing (remote) file. The second second argument should be the
  # new name of that file, or its full (remote) path.
  #
  # Also note that it seems as if the FTP protocol requires the full
  # target location, otherwise errors such as the following might
  # happen:
  #
  #   ftp.rb in `getresp': 451 Rename/move failure: Is a directory (Net::FTPTempError)
  #
  # The command used by .rename() command should be equivalent to this code:
  #
  #   SITE mv oldpath newpath
  #
  # Documentation for the functionality can be found here:
  #
  #   http://ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html#method-i-rename
  #        
  # ========================================================================= #
  def rename(
      from,
      to,
      be_verbose = be_verbose?
    )
    to = to.first if to.is_a? Array
    if from.is_a? Array # Support batch-transfer.
      from.each {|entry| rename(entry, to) }
    else
      from = rds(from)
      to   = rds(to)
      begin
        ftp_object?.rename(from, to)
        if be_verbose
          opnn; e "Renamed the remote entry `#{sfancy(from)}"\
                  "` to `#{sfancy(to)}`."
        end
      rescue Net::FTPTempError => error
        e "An error (#{simp('Net::FTPTempError')}) occurred "\
          "in the method #{simp('rename()')}:"
        pp error
      end
    end
  end; alias rename_file        rename # === rename_file
       alias move_file          rename # === move_file
       alias rename_remote_file rename # === rename_remote_file

  # ========================================================================= #
  # === update_remote_file_listing
  # ========================================================================= #
  def update_remote_file_listing(
      be_verbose = be_verbose?
    )
    case be_verbose
    when :be_quiet
      be_verbose = false
    end
    if is_connected?
      begin
        # =================================================================== #
        # Make use of a method that will skip a few entries, such as
        # "." or "..".
        # =================================================================== #
        @internal_hash[:raw_entries] = return_slightly_sanitized_entries_ignoring_a_few
      rescue Exception => error
        opnn; e swarn('An error happened in the method '\
                'update_remote_file_listing(), at '\
                'line: '+__LINE__.to_s+'.')
        pp error.class
        pp error
      end
    else
      if be_verbose
        opnn; e 'Can not update the remote file listing '\
                'because we are not connected.'
      end
    end
  end; alias update_file_listing update_remote_file_listing # === update_file_listing
       alias update_raw_listing  update_remote_file_listing # === update_raw_listing
       alias get_listing         update_remote_file_listing # === get_listing
       alias get_remote_listing  update_remote_file_listing # === get_remote_listing
       alias update              update_remote_file_listing # === update

  # ========================================================================= #
  # === does_this_remote_file_exist?
  # ========================================================================= #
  def does_this_remote_file_exist?(i)
    i = File.basename(i).strip
    array = return_all_remote_files
    if array.is_a?(Array) and
       array.first.is_a?(String)
      array.map! {|entry|
        entry = FtpParadise::Entry.new(entry)
      }
      array.select! {|entry| entry.is_a_file? }
      array.map! {|entry| entry.filename? }
    end
    return array.include?(i)
  end

end; end