#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/misc.rb'
# =========================================================================== #
module FtpParadise

class Connection

  require 'ftp_paradise/connection/directory_handling.rb'
  require 'ftp_paradise/connection/download.rb'
  require 'ftp_paradise/toplevel_methods/clear_user_dataset.rb'

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    FtpParadise.data?
  end

  # ========================================================================= #
  # === raw_list
  # ========================================================================= #
  def raw_list(i = '*')
    ftp_object?.list(i)
  end

  # ========================================================================= #
  # === return_slightly_sanitized_entries_ignoring_a_few
  # ========================================================================= #
  def return_slightly_sanitized_entries_ignoring_a_few(
      i = '*'
    )
    ftp_object?.list(i).reject {|entry|
      entry.to_s.strip.empty? or entry.end_with?(' .') or entry.end_with?(' ..')
    }.map {|line|
      if line.end_with? ':' # <- Some remote FTP-hosts use ':' for a directory.
        line[-1,1] = '/'
      end
      line
    }
  end; alias list return_slightly_sanitized_entries_ignoring_a_few # === list

  # ========================================================================= #
  # === nlst?
  #
  # Returns an array of filenames in the remote directory. This will
  # yield less information than the "ls -l" variant, aka list().
  #
  # Documentation:
  #
  #   https://ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html#method-i-nlst
  #
  # ========================================================================= #
  def nlst?
    ftp_object?.nlst
  end; alias nlst nlst? # === nlst

  # ========================================================================= #
  # === list?                                            (list tag)
  #
  # Returns an array of file information in the directory (the output
  # is like `ls -l`). If a block is given, it iterates through the
  # listing.
  # ========================================================================= #
  def list?(i = '*')
    # ======================================================================= #
    # We will also sanitize this a bit.
    # ======================================================================= #
    @internal_hash[:raw_entries] = return_slightly_sanitized_entries_ignoring_a_few
    return @internal_hash[:raw_entries] # And return the result here.
  end; alias ls                   list? # === ls
       alias populate_raw_entries list? # === populate_raw_entries

  # ========================================================================= #
  # === passive=
  # ========================================================================= #
  def passive=(i = true)
    if ftp_object?
      ftp_object?.passive = i
    else
      e 'No ftp-object has been instantiated yet.'
    end
  end

  # ========================================================================= #
  # === list_content                                       (list tag, ls tag)
  #
  # List available data.
  # ========================================================================= #
  def list_content
    report_current_remote_dir
    result = list?
    cliner { pp result }
    _ = result.first[0, 1]
    case _
    when 'd'
      e "Directory #{result.first}"
      # @array_directories
    when 'l'
      e "Link #{result.first}"
    else
      e "File #{result.first}"
    end
  end; alias do_list_content list_content # === list
       alias listing?        list_content # === listing?

  # ========================================================================= #
  # === file_listing_as_entries?
  # ========================================================================= #
  def file_listing_as_entries?
    _ = remote_file_listing?
    if _
      return _.map {|entry| FtpParadise::Entry.new(entry) }
    end
    return nil
  end

  # ========================================================================= #
  # === empty?
  # ========================================================================= #
  def empty?
    array_remote_files.empty?
  end

  # ========================================================================= #
  # === find_matches_for
  #
  # This method will return an array of files and directories that match
  # to the provided input (which we will assume to be a query).
  # ========================================================================= #
  def find_matches_for(i)
    i = i.delete '*'
    update_file_listing
    _ = return_remote_directory_content.map(&:first)
    _ = _.grep(/#{i}/)
    return _
  end

  # ========================================================================= #
  # === output_sorted_by_time
  #
  # This method will output the remote content in a time-sorted manner.
  # ========================================================================= #
  def output_sorted_by_time
    _ = sanitized_remote_directory_content?
    sorted_result = _.sort_by {|array|
      array.last
    }.reverse
    sorted_result.each_with_index {|array, index| index += 1
      file_or_directory = array[1]
      name_of_the_file = array.first
      if file_or_directory == 'directory'
        name_of_the_file << '/' unless name_of_the_file.end_with? '/'
      end
      index = simp( (index.to_s+') ').rjust(6))
      e index+sfancy(name_of_the_file)
    }
  end

  # ========================================================================= #
  # === last_response
  #
  # The server's last response is available through this method here.
  # ========================================================================= #
  def last_response
    ftp_object?.last_response
  end; alias last_response?        last_response # === last_response?
       alias last_server_response? last_response # === last_server_response?

  # ========================================================================= #
  # === raw_entries?
  # ========================================================================= #
  def raw_entries?
    @internal_hash[:raw_entries]
  end; alias remote_file_listing? raw_entries? # === remote_file_listing?
       alias file_listing?        raw_entries? # === file_listing?
       alias raw_listing?         raw_entries? # === raw_listing?

  # ========================================================================= #
  # === return_all_remote_files
  #
  # This method will return an Array with all remote files.
  # ========================================================================= #
  def return_all_remote_files
    results = nil
    if file_listing?
      results = file_listing?.select {|entry|
        if entry.is_a?(String) and !entry.empty?
          entry = FtpParadise::Entry.new(entry)
        end
        entry.is_a_file?
      }
    end
    return results
  end; alias remote_file_listing?       return_all_remote_files # === remote_file_listing?
       alias all_remote_files?          return_all_remote_files # === all_remote_files?
       alias array_remote_files         return_all_remote_files # === array_remote_files
       alias return_remote_files        return_all_remote_files # === return_remote_files
       alias return_remote_file_listing return_all_remote_files # === return_remote_file_listing
       alias remote_files?              return_all_remote_files # === remote_files
       alias files?                     return_all_remote_files # === files?

  # ========================================================================= #
  # === status?
  #
  # Report the status here. This is equivalent to the STAT command.
  # ========================================================================= #
  def status?
    ftp_object?.status.to_s
  end; alias status status? # === status

  # ========================================================================= #
  # === return_local_directories
  #
  # Return all local directories.
  # ========================================================================= #
  def return_local_directories
    Dir['*'].select {|entry| File.directory?(entry) }
  end

  # ========================================================================= #
  # === modification_time_of?
  #
  # This method will give us the last modification time of a remote
  # file, by issuing the mdtm() command.
  #
  # The format returned will be in YYYMMDDhhmmss.
  #
  # If you require a parsed Time instance, you can use mtime()
  # on the ftp object.
  #
  # Symbols could also be used, such as "pp modification_time_of? :a".
  # ========================================================================= #
  def modification_time_of?(i)
    if i.is_a? Array
      i.each {|entry| modification_time_of(entry) } # <- Recursive call.
    else
      ftp_object?.mdtm(i.to_s)
    end
  end; alias mdtm modification_time_of? # === mdtm

  # ========================================================================= #
  # === set_file
  #
  # Use only this to modify the @file variable.
  # ========================================================================= #    
  def set_file(i = nil)
    @internal_hash[:file] = i
  end

  # ========================================================================= #
  # === is_file?
  #
  # This method will check whether the input is a file or not.
  # It will call the method .is_directory?()
  # ========================================================================= #
  def is_file?(i)
    !is_a_directory?(i)
  end

  # ========================================================================= #
  # === logged_in?
  # ========================================================================= #
  def logged_in?
    ftp_object?.instance_variable_get '@logged_in'
  end

  # ========================================================================= #
  # === available_hosts?
  # ========================================================================= #
  def available_hosts?
    @internal_hash[:array_available_hosts]
  end; alias array_available_hosts  available_hosts? # === array_available_hosts
       alias array_available_hosts? available_hosts? # === array_available_hosts?

  # ========================================================================= #
  # === open_timeout?
  # ========================================================================= #
  def open_timeout?
    ftp_object?.open_timeout.to_s if ftp_object?.respond_to? :open_timeout
  end

  # ========================================================================= #
  # === feedback_login_hosts
  #
  # Feedback all available login hosts, based on the information stored
  # in the Array @array_available_hosts.
  # ========================================================================= #
  def feedback_login_hosts
    opnn; e 'The followings hosts are easily available:'
    e
    available_hosts?.each { |host| e "  - #{sfancy(host)}" }
    e
  end

  # ========================================================================= #
  # === help?
  # ========================================================================= #
  def help?
    ftp_object?.help
  end; alias help help? # === help

  # ========================================================================= #
  # === close                                                     (close tag)
  #
  # Close the connection here.
  #
  # Further operations will be impossible until a new connection
  # is opened up again, via connect().
  # ========================================================================= #
  def close
    ftp_object?.close
  end

  # ========================================================================= #
  # === mlsd
  # ========================================================================= #
  def mlsd(i)
    ftp_object?.mlsd(i)
  end

  # ========================================================================= #
  # === site
  # ========================================================================= #
  def site(i = nil)
    ftp_object?.site(i)
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @internal_hash[:file]
  end

  # ========================================================================= #
  # === system_command
  #
  # This will return system information from the remote host.
  #
  # Stuff like:
  #   UNIX Type: L8
  # ========================================================================= #
  def system_command
    ftp_object?.system
  end; alias system system_command # === system

  # ========================================================================= #
  # === report_host_port_user_name_and_password
  # ========================================================================= #
  def report_host_port_user_name_and_password
    ljust = 15
    e '- Now trying to log in to '+simp(host?)+
      ' (Port: '+swarn(port?)+') using:'
    e '  '+('Login Name: ').ljust(ljust)+sfancy(user_name?)
    e '  '+('Password: ').ljust(ljust)+sfancy(password?)
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === read_timeout=
  #
  # Setter-method for the #read_timeout attribute.
  # ========================================================================= #
  def read_timeout=(i)
    ftp_object?.read_timeout = i
  end

  # ========================================================================= #
  # === size?
  #
  # This method will return the size of the given (remote) filename.
  # ========================================================================= #
  def size?(i)
    ftp_object?.size(i)
  end; alias size size? # === size

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    @internal_hash[:padding]
  end

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @internal_hash[:be_verbose]
  end

  # ========================================================================= #
  # === server_welcome_message?
  #
  # This method will return the remote server's welcome message.
  # ========================================================================= #
  def server_welcome_message?
    ftp_object?.welcome
  end; alias welcome? server_welcome_message? # === welcome?
       alias welcome  server_welcome_message? # === welcome

  # ========================================================================= #
  # === noop
  # ========================================================================= #
  def noop
    ftp_object?.noop
  end

  # ========================================================================= #
  # === last_response_code?
  #
  # Return the server's last response code.
  # ========================================================================= #
  def last_response_code?
    ftp_object?.last_response_code
  end; alias last_response_code last_response_code? # === last_response_code

  # ========================================================================= #
  # === quit                                                       (quit tag)
  #
  # Use this to quit (and disconnect) from the FTP Connection again.
  # ========================================================================= #
  def quit
    begin
      if is_connected?
        if ftp_object?
          FtpParadise.clear_user_dataset
          ftp_object?.quit
        end
      end
    rescue Exception => error
      opnn; e "An error happened in the method #{sfancy(__method__)}()"
      pp error
    end
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def [](i)
    update_file_listing
    if i =~ /^\d+$/ # If is a number.
      i = remote_file_listing.size if i.to_i > remote_file_listing.size
      i = remote_file_listing[i.to_i - 1].first
    end
    return i
  end

  # ========================================================================= #
  # === sendcmd
  #
  # Sends a command and returns the response of the server.
  #
  # This has to be rescued, because there are lots of different errors
  # possible.
  # ========================================================================= #
  def sendcmd(i)
    begin
      ftp_object?.sendcmd(i)
    rescue Exception => error
      pp error
      e '(This error may be due to the remote file not existing.)'
    end
  end

end; end