#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/remove.rb'
# =========================================================================== #
module FtpParadise

class Connection

  # ========================================================================= #
  # === remove                                                   (remove tag)
  # ========================================================================= #
  def remove(i)
    if i.is_a? Array
      i.each {|entry| remove(entry) }
    else
      if is_a_directory?(i)
        remove_remote_directory(i)
      elsif is_a_file?(i)
        remove_remote_file(i, be_verbose?)
      end
    end
  end

  # ========================================================================= #
  # === remove_remote_file
  #
  # Remove a (remote) file, via the delete() method of the FTP protocol.
  #
  # Note that this method will only delete files.
  #
  # The official documentation for the Ruby-FTP delete() method can be
  # found here:
  #
  #   https://www.ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html#method-i-delete
  #
  # ========================================================================= #
  def remove_remote_file(
      i,
      be_verbose = be_verbose?
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    if i.is_a? Array
      i.each {|entry| remove_remote_file(entry, be_verbose) }
    else
      if does_this_remote_file_exist?(File.basename(i))
        i = File.basename(i)
        if i =~ /^\d+$/ # if is a number
          update_file_listing
          _ = return_remote_file_listing[i.to_i - 1].first
          _ = File.basename(_)
          e "Performing a substitution of #{sfancy(i)}"\
            " to #{simp(_)}."
          i = _
        end
        if be_verbose
          notify_the_user_about(i, __method__)
        end
        ftp_object?.delete(i)
        update_file_listing
        return i # Also return the file that was deleted.
      else
        opnn; e "No remote file called `#{sfile(i)}` was "\
                "found at `#{sfancy(remote_pwd)}`."
      end
    end
  end; alias remote_remove       remove_remote_file # === remote_remove
       alias remote_remove_files remove_remote_file # === remote_remove_files
       alias remote_remove_file  remove_remote_file # === remote_remove_files
       alias delete              remove_remote_file # === delete
       alias remove_file         remove_remote_file # === remove_file
       alias rm_file             remove_remote_file # === rm_file
       alias rmfile              remove_remote_file # === rmfile

  # ========================================================================= #
  # === original_delete
  #
  # This method will remove the file, without doing any additional
  # checks.
  # ========================================================================= #
  def original_delete(i)
    ftp_object?.delete(i)
  end

  # ========================================================================= #
  # === remove_remote_directory
  #
  # This method will remove a remote directory. For this to work, as
  # operation, the remote directory has to exist.
  #
  # This operation may fail e. g. when the remote directory is not empty.
  # This explains why we attempt to rescue some errors.
  # ========================================================================= #
  def remove_remote_directory(
      i,
      be_verbose   = be_verbose?,
      force_remove = false
    )
    case force_remove
    when :force_remove
      force_remove = true
    when :do_not_force_remove
      force_remove = false
    end
    if i.is_a? Array
      i.each {|entry| remove_remote_directory(entry, be_verbose) }
    else
      if be_verbose
        notify_the_user_about(i, __method__)
      end
      if is_a_directory? i
        begin
          ftp_object?.rmdir(i)
        rescue Net::FTPPermError => error
          opnn; e swarn('We could not remove the remote directory at ')+
                  sdir(i)+swarn(' as it is not empty.')
          pp error
          if force_remove
            opnn; e 'We will now try to enter into this directory'
            opnn; e 'and then remove all files of that directory.'
            rcd(this_dir)
            update_file_listing
            all_remote_files?.each {|file| remove(file) } # Now it should be removed.
            rcd('..') # Go back again.
            rmdir(i, be_verbose, :do_not_force_remove) # But we won't force again, to avoid recursive loops.
            opnn; e "The remote directory at #{sdir(i)} has been removed."
          end
        end
      else
        if be_verbose
          opnn; e "The given input #{sfancy(i)} is not a directory."
        end
      end
    end
  end; alias rmdir                           remove_remote_directory # === rmdir
       alias remove_these_remote_directories remove_remote_directory # === remove_these_remote_directories
       alias remove_directory                remove_remote_directory # === remove_directory

end; end