#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/show.rb'
# =========================================================================== #
module FtpParadise

class Connection

  # ========================================================================= #
  # === show_remote_directory_listing
  #
  # This will simply show the content of a remote directory.
  #
  # Invocation examples:
  #
  #   show_remote_directory_listing
  #   show_remote_directory_listing :default, :show_only_directories
  #
  # ========================================================================= #
  def show_remote_directory_listing(
      i             = list,
      array_options = []
    )
    unless array_options.is_a? Array
      array_options = [array_options]
    end
    case i
    when :default
      i = list
    end
    if i.is_a? Array
      # ===================================================================== #
      # Get rid of '.' and '..'.
      # ===================================================================== #
      i.reject! {|entry| entry.end_with?('..') or entry.end_with?('.') }
      # ===================================================================== #
      # First, turn them into entries.
      # ===================================================================== #
      i.map! {|entry| FtpParadise::Entry.new(entry) }
      # ===================================================================== #
      # Next, we can use different behaviours.
      # ===================================================================== #
      if array_options.include? :show_only_directories
        i.select! {|entry| entry.is_a_directory? }
      end
      i.each {|entry| entry.show_the_line }
    end
  end; alias show_remote_files show_remote_directory_listing # === show_remote_files

  # ========================================================================= #
  # === show_full_names?
  # ========================================================================= #
  def show_full_names?
    @internal_hash[:show_full_names]
  end

  # ========================================================================= #
  # === show_remote_listing
  #
  # Use this method whenever you want to show the remote entries.
  #
  # As of June 2016, you can pass in your own dataset into this method
  # as well. It must conform to the Array with the four entries, which
  # is also returned via the method sanitized_remote_directory_content?.
  # ========================================================================= #
  def show_remote_listing(
      dataset    = sanitized_remote_directory_content?,
      be_verbose = be_verbose?
    )
    n_rjust = 10
    _ = dataset
    if show_full_names?
      _.map! {|entry|
        filename = entry[0]
        entry[0] = remote_pwd?+filename # This is the file size.
        entry
      }
    end
    _.each_with_index { |array, index|
      if be_verbose # Display only when in verbose mode, which is the default.
        file_or_directory = array[1]
        name_of_the_file = array.first
        # =================================================================== #
        # Get a pointer to the filesize as well.
        # =================================================================== #
        filesize = array[2].to_s.rjust(n_rjust)
        if file_or_directory == 'directory'
          name_of_the_file << '/' unless name_of_the_file.end_with? '/'
        end
        # =================================================================== #
        # === Put a proper padding to the index
        # =================================================================== #
        new_index = (index+1).to_s.rjust(_.size.to_s.size)
        n_pad = 125
        case file_or_directory
        when 'directory' # Display for directories.
          name_of_the_file = sdir(name_of_the_file)
          e (padding?+sdir(new_index+') ')+name_of_the_file).ljust(n_pad)+' '+
            filesize
        else
          e (padding?+sfancy(new_index+') ')+sfancy(name_of_the_file)).ljust(n_pad)+' '+
            filesize
        end
      end
    }
  end; alias show_file_listing        show_remote_listing # === show_file_listing
       alias show_remote_file_listing show_remote_listing # === show_remote_file_listing

  # ========================================================================= #
  # === show_remote_directories
  # ========================================================================= #
  def show_remote_directories
    show_remote_directory_listing :default, :show_only_directories
  end

  # ========================================================================= #
  # === show_full_names=
  # ========================================================================= #
  def show_full_names=(i = true)
    @internal_hash[:show_full_names] = i
  end; alias show_full_names show_full_names= # === show_full_names

  # ========================================================================= #
  # === show_user_and_password
  # ========================================================================= #
  def show_user_and_password
    ljust = 12
    e
    e ('User-Name: ').ljust(ljust)+sfancy(user_name?.to_s)
    e ('Password: ' ).ljust(ljust)+sfancy(password?)
    e
  end; alias show_user_name_and_password show_user_and_password # === show_user_name_and_password

  # ========================================================================= #
  # === show_host_user_name_port_and_password
  #
  # Inform us about the basic settings.
  # ========================================================================= #
  def show_host_user_name_port_and_password(
      do_use_spacer_around_the_output = true
    )
    ljust = 15
    e if do_use_spacer_around_the_output
    e ('Host: '     ).ljust(ljust)+sfancy(host?)
    e ('User-Name: ').ljust(ljust)+sfancy(user_name?.to_s)
    e ('Port: '     ).ljust(ljust)+sfancy(port?)
    e ('Password: ' ).ljust(ljust)+sfancy(password?)
    e if do_use_spacer_around_the_output
  end

end; end