#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/transfer_mode.rb'
# =========================================================================== #
module FtpParadise

class Connection

  # ========================================================================= #
  # === set_passive_mode
  #
  # Use this when modifying the passive setting.
  # ========================================================================= #
  def set_passive_mode
    set_transfer_mode :ascii
    ftp_object?.passive = true if ftp_object?
  end; alias set_passive set_passive_mode # === set_passive

  # ========================================================================= #
  # === set_default_transfer_mode
  # ========================================================================= #
  def set_default_transfer_mode(
      be_verbose = :be_quiet
    )
    set_transfer_mode(DEFAULT_TRANSFER_MODE, be_verbose)
  end

  # ========================================================================= #
  # === set_transfer_mode
  #
  # This method will set to either one of two different possibles modes:
  #
  #   (1) :ascii
  #   (2) :binary
  #   (3) :passive
  #
  # Note that :passive "mode" is ascii, with an additional invocation to
  # change the transfer mode to .passive.
  #
  # It will also change passive mode. The argument :default will default
  # to the more commonly used ASCII mode.
  #
  # When would you want to use ASCII mode?
  #
  # Use ascii mode for:
  #
  #   txt, rtf, html, php
  #
  # Use binary mode for:
  #
  #   images, videos and similar.
  #
  # ========================================================================= #
  def set_transfer_mode(
      i          = DEFAULT_TRANSFER_MODE, # Will usually default to :ascii
      be_verbose = be_verbose?
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    when :be_quiet
      be_verbose = false
    end
    i = :ascii if i == :default
    case i
    # ======================================================================= #
    # === :ascii
    # ======================================================================= #
    when :ascii
      notify_the_user_about(i, __method__) if be_verbose
      @internal_hash[:transfer_mode] = :ascii
      ftp_object?.binary = false if ftp_object?
    # ======================================================================= #
    # === :binary
    # ======================================================================= #
    when :binary
      notify_the_user_about(i, __method__) if be_verbose
      @internal_hash[:transfer_mode] = :binary
    # ======================================================================= #
    # === :passive
    # ======================================================================= #
    when :passive,
         :pass
      notify_the_user_about(i, __method__) if be_verbose
      @internal_hash[:transfer_mode] = :ascii
      ftp_object?.passive = true if ftp_object?
    else
      opnn; e swarn('Not found the following mode: ')+
              sfancy(i.to_s)
    end
  end; alias mode_to_use= set_transfer_mode # === mode_to_use=
       alias set_mode     set_transfer_mode # === set_mode

  # ========================================================================= #
  # === report_transfer_mode_in_use
  # ========================================================================= #
  def report_transfer_mode_in_use
    e "Transfer mode: #{sfancy(transfer_mode?.to_s)}"
  end

  # ========================================================================= #
  # === set_active
  # ========================================================================= #
  def set_active
    ftp_object?.passive = false
  end
  
  # ========================================================================= #
  # === passive_transfer?
  # ========================================================================= #
  def passive_transfer?
    ftp_object?.passive
  end; alias passive? passive_transfer? # === passive?

  # ========================================================================= #
  # === transfer_mode?
  #
  # This method will return the currently used transfer_mode. This mode
  # can be either :binary, or :ascii. By default it will be :binary.
  #
  # The transfer mode can be :binary or :text. :ascii is an alias to
  # :text.
  # ========================================================================= #
  def transfer_mode?
    @internal_hash[:transfer_mode]
  end; alias transfer_mode transfer_mode? # === transfer_mode
       alias mode?         transfer_mode? # === mode?
       alias type          transfer_mode? # === type
       alias type?         transfer_mode? # === type?

  # ========================================================================= #
  # === set_ascii
  #
  # Easier wrapper method.
  # ========================================================================= #
  def set_ascii
    set_transfer_mode :ascii
  end; alias set_ascii_mode set_ascii # === set_ascii_mode

  # ========================================================================= #
  # === binary?
  #
  # When this is true, then then all FTP-transfers will be performed in
  # binary mode.
  #
  # Deaults to true.
  # ========================================================================= #
  def binary?
    ftp_object?.binary
  end; alias binary binary? # === binary

  # ========================================================================= #
  # === set_binary_mode
  # ========================================================================= #
  def set_binary_mode
    set_transfer_mode :binary
    ftp_object?.binary = true
  end; alias set_binary         set_binary_mode # === set_binary
       alias set_to_binary_mode set_binary_mode # === set_to_binary_mode

end; end