#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/connection/upload.rb'
# =========================================================================== #
module FtpParadise
        
class Connection

  require 'ftp_paradise/connection/remote_pwd.rb'
  require 'ftp_paradise/connection/is_connected.rb'

  # ========================================================================= #
  # === upload_binary_file
  #
  # Use this method to upload a binary file to a remote host.
  #
  # The first argument `i` is the file that we wish to upload.
  #
  # The method will make use of the method .putbinaryfile().
  #
  # Documentation for that file can be found here:
  #
  #   https://ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html#method-i-putbinaryfile
  #
  # ========================================================================= #
  def upload_binary_file(
      i,
      be_verbose = be_verbose?
    )
    be_verbose = false if be_verbose == :be_silent
    if i.is_a? Array
      i.each {|entry| upload_binary_file(entry, be_verbose) }
    else
      i = i.strip.delete(N) # Added Feb 2014.
      if be_verbose
        notify_the_user_about(i, __method__)
      end
      begin
        # =================================================================== #
        # Next we will delegate towards the @ftp_object:
        # =================================================================== #
        ftp_object?.putbinaryfile(i) # We will use the method .putbinaryfile()
        @internal_hash[:uploaded_these_files] << i
        if be_verbose
          remote_path = rds(remote_path?+'/'+File.basename(i))
          e 'Done uploading! The remote path is: '
          e "  #{simp(remote_path)}"
          report_finished_uploading_of_this_file(i)
        end
      rescue Exception => error
        e 'An error happened as we tried to use the method '+simp('upload_binary()')+'.'
        e 'The method we used was: '+simp('ftp?.putbinaryfile(i)')
        e 'We will feedback this error now (of class '+sfancy(error.class.to_s)+'):'
        pp error
      end
    end
  end; alias upload_binary           upload_binary_file # === upload_binary
       alias upload_this_binary_file upload_binary_file # === upload_this_binary_file()

  # ========================================================================= #
  # === upload_file
  #
  # This method will distinguish between a binary file or a text file,
  # based on the transfer mode in use. The default is to work on a
  # text file.
  # ========================================================================= #
  def upload_file(
      i,
      be_verbose = be_verbose?
    )
    if File.exist? i
      case transfer_mode?
      # ===================================================================== #
      # === :ascii
      # ===================================================================== #
      when :ascii,
           :text,
           :default
        upload_this_text_file(i, be_verbose)
      # ===================================================================== #
      # === :binary
      # ===================================================================== #
      when :binary
        upload_this_binary_file(i, be_verbose)
      end
      update_raw_listing(:be_quiet)
    else
      opnn; e "No file called `#{sfile(i)}` appears to exist locally." 
    end
  end; alias upload_this_file upload_file # === upload_this_file

  # ========================================================================= #
  # === upload                                                   (upload tag)
  #
  # This method can upload binary files, text files and whole directories.
  # Based on that check, the method body will behave differently, and
  # delegate to other methods such as upload_directory() or upload_file().
  #
  # A single file (as String) can be uploaded, or an Array of files. The
  # upload activity will transfer the file or the files onto the remote host.
  #
  # The second argument specifies whether the method will be verbose and
  # report back to the user, or whether the method will be quiet,
  # defaulting to the return value of the method `be_verbose?`.
  # ========================================================================= #
  def upload(
      i,
      be_verbose = be_verbose?
    )
    be_verbose = false if be_verbose == :be_silent
    if i.is_a? Array
      i.each {|entry| upload(entry, be_verbose) }
    else
      if i == '*' # Here we want to upload all files.
        i = get_all_local_files
        upload(i, be_verbose) 
      else
        # =================================================================== #
        # === Handle upload of directories first
        # =================================================================== #
        if File.directory? i
          upload_directory(i, be_verbose)
        # =================================================================== #
        # === Handle uploading of files next
        # =================================================================== #
        else
          upload_file(i, be_verbose)
        end
      end
    end
  end

  # ========================================================================= #
  # === uploaded_these_files?
  # ========================================================================= #
  def uploaded_these_files?
    @internal_hash[:uploaded_these_files]
  end

  # ========================================================================= #
  # === report_finished_uploading_of_this_file
  #
  # Report that we have finished uploading a file.
  #
  # We should also denote the leading http part.
  # ========================================================================= #
  def report_finished_uploading_of_this_file(i)
    if is_connected?
      remote_path = "http://#{rds(remote_path?+File.basename(i))}"
      opnn; e "Done uploading `#{sfile(i)}` to `"\
              "#{sdir(remote_path)}`!"
    end
  end

  # ========================================================================= #
  # === puttextfile
  #
  # This method will transfer a localfile to the remote server in
  # ASCII (text) mode. The result will be stored in "remotefile".
  #
  # If a callback or an associated block is supplied, calls it, passing
  # in the transmitted data one line at a time.
  # ========================================================================= #
  def puttextfile(
      local_file, remotefile = File.basename(local_file)
    )
    begin
      ftp_object?.puttextfile(local_file)
      @internal_hash[:uploaded_these_files] << local_file
    rescue Net::FTPPermError => error
      opnn; e swarn('Can not upload `')+sfile(local_file)+swarn('`.')
      opnn; e swarn('Reason provided to this method was:')
      opnn; e "  → #{sfancy(error.to_s)}"
    rescue Exception => error
      opnn; e "An error happened in the method #{__method__}"
      pp error
    end
  end

  # ========================================================================= #
  # === upload_this_text_file
  #
  # This method can be used to upload one or several text files.
  #
  # The first argument should be an Array or a String, denoting the
  # name/path of the local text file that you wish to upload.
  # ========================================================================= #
  def upload_this_text_file(
      i,
      be_verbose = be_verbose?
    )
    be_verbose = false if be_verbose == :be_silent
    if i.is_a? Array
      i.each {|entry| upload_this_text_file(entry, be_verbose) }
    else
      i = i.strip.delete(N) # Added in February 2014.
      case i
      when :default
        ENV['MISC'].to_s+'/SITEMAP.cgi' # <- Makes only sense for text-files anyway.
      end
      case be_verbose
      when :be_quiet
        be_verbose = false
      end
      if be_verbose
        notify_the_user_about(i, __method__)
      end
      puttextfile(i) # <- Delegate towards the method puttextfile().
      report_finished_uploading_of_this_file(i) if be_verbose
    end
  end

  # ========================================================================= #
  # === upload_this_directory
  #
  # This method can be used to upload a local directory to a remote host.
  #
  # If the first input argument is an Array, then the method will call
  # itself recursively. This allows us to transfer several directories
  # in one go.
  #
  # If the input is a (local and existing) directory, then this method
  # will also have to create a remote directory, then chdir into it,
  # and then upload the content of that local directory to the remote
  # host.
  # ========================================================================= #
  def upload_this_directory(
      i,
      be_verbose = be_verbose?
    )
    if i.is_a? Array
      i.each {|entry| upload_this_directory(i, be_verbose) }
    else
      if File.directory? i
        unless this_remote_directory_exists?(i)
          remote_create_directory(i, be_verbose)
        end
        change_local_directory(i)
        change_remote_directory(i)
        report_remote_directory
        content = return_directory_content(i) # Last but not least, copy all files to that dir.
        content.each {|this_file| upload(this_file) }
      else
        opnn; e "The given input `#{sfancy(i)}` is not a directory."
        opnn; e 'This method can only upload directories.'
      end
    end
  end; alias upload_directory         upload_this_directory # === upload_directory
       alias upload_these_directories upload_this_directory # === upload_these_directories

end; end