#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/constants/roebe_ftp_constants.rb'
# include FtpParadise::RoebeFtpConstants
# =========================================================================== #
module FtpParadise # === FtpParadise::RoebeFtpConstants

module RoebeFtpConstants

  require 'ftp_paradise/constants/roebe.rb'
  require 'yaml'

  # ========================================================================= #
  # === FtpParadise::RoebeFtpConstants::@hash_ftp_dataset
  # ========================================================================= #
  @hash_ftp_dataset = {}

  # ========================================================================= #
  # === FTP_DATASET
  #
  # Here we will load my custom ftp file, but only if this file exists.
  #
  # If it does not exist, then we will not load it, and we will not
  # make use of it. That way others can ignore this file altogether.
  #
  # The location for my file is at:
  #
  #   bl $MY_FTP/YAML/roebe_ftp.yml
  #
  # ========================================================================= #
  if File.exist? FILE_ROEBE_FTP
    FTP_DATASET = FILE_ROEBE_FTP
  else
    begin
      home_dir = File.expand_path('~')
    rescue; end
    home_dir = '/root' if home_dir.nil? # A safeguard here if ENV is unavailable.
    FTP_DATASET = "#{home_dir}/roebe_ftp.yml"
  end

  # ========================================================================= #
  # === FtpParadise::RoebeFtpConstants.file?
  #
  # Feedback where that file is.
  # ========================================================================= #
  def self.file?
    FILE_ROEBE_FTP
  end

  # ========================================================================= #
  # === FtpParadise::RoebeFtpConstants.hash_ftp_dataset?
  # ========================================================================= #
  def self.hash_ftp_dataset?
    @hash_ftp_dataset
  end; self.instance_eval { alias hash? hash_ftp_dataset? } # === FtpParadise::RoebeFtpConstants.hash_ftp_dataset?.hash?

  # ========================================================================= #
  # Next check whether the roebe_ftp.yml file exists. If it does then
  # we will load it up.
  # ========================================================================= #
  if File.exist? file?
    dataset = YAML.load_file(file?)
    @hash_ftp_dataset.update(dataset) # Sync the dataset.
    # ======================================================================= #
    # The format of the dataset has to be uniform. We start with the
    # remote URL, then the user name, the password, and then the port.
    # ======================================================================= #

    # ======================================================================= #
    # === A1 TAG
    # ======================================================================= #
    _ = dataset['a1']
    A1 = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === ZYMIX TAG
    # ======================================================================= #
    _ = dataset['zymix']
    ZYMIX = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === Bytehost Tag
    #
    # bl $MY_FTP/YAML/roebe_ftp.yml
    # ======================================================================= #
    _ = dataset['bytehost']
    BYTEHOST = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]
    
    # ======================================================================= #
    # === UNIVIE
    #
    # Uni tag, Univie tag
    # ======================================================================= #
    _ = dataset['univie']
    UNIVIE = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === Zerofees tag
    # ======================================================================= #
    _ = dataset['zerofees']
    ZEROFEES = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === UCOZ tag
    # ======================================================================= #
    _ = dataset['ucoz']
    UCOZ = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === UNIWIEN_HOMEPAGE tag
    # ======================================================================= #
    _ = dataset['uniwien_homepage']
    UNIWIEN_HOMEPAGE = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === bplaced
    # ======================================================================= #
    _ = dataset['bplaced']
    BPLACED = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === podserver
    # ======================================================================= #
    _ = dataset['podserver']
    PODSERVER = [
      _['url'],
      _['user_name'],
      _['password'],
      _['port']
    ]

    # ======================================================================= #
    # === RoebeFtpConstants::ARRAY_AVAILABLE_HOSTS
    #
    # All available hosts are kept in the following Array.
    #
    # It is always available when the roebe_ftp.yml file was found.
    #
    # Append to this Array if you add new entries above.
    # ======================================================================= #
    ARRAY_AVAILABLE_HOSTS = [
      BYTEHOST.first,
      A1.first,
      ZYMIX.first,
      UNIVIE.first,
      ZEROFEES.first,
      UCOZ.first,
      UNIWIEN_HOMEPAGE.first,
      PODSERVER.first
    ]
  end

  # ========================================================================= #
  # === RoebeFtpConstants.possible_hosts?
  # ========================================================================= #
  def self.possible_hosts?
    ARRAY_AVAILABLE_HOSTS
  end

  # ========================================================================= #
  # === RoebeFtpConstants.geas?
  # ========================================================================= #
  def self.geas?
    GEAS
  end

  # ========================================================================= #
  # === RoebeFtpConstants.return_user_name_based_on_given_host
  #
  # This method may return the proper user name.
  # ========================================================================= #
  def self.return_user_name_based_on_given_host(i)
    if ARRAY_AVAILABLE_HOSTS.include? i
      i = PODSERVER[1] # Hardcoded for now. [1] is the user-name.
    end if FtpParadise.const_defined? :RoebeFtpConstants
    i
  end

end; end

if __FILE__ == $PROGRAM_NAME # Testing RoebeFtpConstants next.
  require 'cliner'
  require 'colours/colours_e_autoinclude.rb'
  include RoebeFtpConstants
  cliner
  e RoebeFtpConstants.file?
  cliner
  pp RoebeFtpConstants.hash_ftp_dataset?
  pp RoebeFtpConstants::GEAS
  pp RoebeFtpConstants.geas?
end # $FTP_PARADISE/roebe_ftp_constants.rb