#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === FtpParadise::Entry
#
# This class is sort of similar to class Net::FTP::List::Entry.
#
# We want to parse a FTP entry into corresponding slots on the object.
#
# The input should be an individual line from a remote Net::FTP object.
#
# The output would be an object such as:
#
#   <FtpParadise::Entry:0x000055561f61b370 @basename="usher-0.0.12.gem", @filesize=2048, @mtime=2014-10-25 00:00:00 +0200, @raw="-rw-r--r--    1 330        330              6144 Dec 27  2015 usher-0.0.12.gem", @server_type="Unix", @symlink=false, @dir=true, @file=false, @device=false, @splitted=["-rw-r--r--", "1", "330", "330", "6144", "Dec", "27", "2015", "usher-0.0.12.gem"]
#
# =========================================================================== #
# require 'ftp_paradise/entry/entry.rb'
# FtpParadise::Entry.new
# =========================================================================== #
module FtpParadise

class Entry # === FtpParadise::Entry

  require 'time'

  alias e puts

  attr_reader :basename
  attr_reader :device
  attr_reader :raw
  attr_reader :server_type

  # ========================================================================= #
  # === FtpParadise::Entry::DEFAULT_INPUT_FOR_THE_ENTRY_CALLED_RAW
  # ========================================================================= #
  DEFAULT_INPUT_FOR_THE_ENTRY_CALLED_RAW = 'drwxr-xr-x    2 330        330              2048 Oct 25  2019 .'
    
  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      input_string = ARGV,
      run_already  = true
    )
    reset
    set_raw(input_string)
    run if run_already
  end

  # ========================================================================= #
  # === set_raw
  # ========================================================================= #
  def set_raw(i)
    i = i.first if i.is_a? Array
    @raw = i.to_s.rstrip
    if @raw.include? ' '
      set_splitted(
        @raw.split(' ')
      ) # Determine @splitted as well here.
    else
      # ===================================================================== #
      # This is for entries that may be short, such as the changed
      # directory-display for FTP data.
      # ===================================================================== #
      @raw = @raw.rjust(67)
      set_splitted(nil)
    end
  end

  # ========================================================================= #
  # === raw_input?
  # ========================================================================= #
  def raw_input?
    @raw
  end; alias raw? raw_input? # === raw?

  # ========================================================================= #
  # === set_splitted
  # ========================================================================= #
  def set_splitted(i)
    @splitted = i
  end

  # ========================================================================= #
  # === last
  #
  # This should return the name of the file.
  # ========================================================================= #
  def last
    @splitted.last
  end

  # ========================================================================= #
  # === symlink?
  # ========================================================================= #
  def symlink?
    @symlink
  end; alias symlink symlink? # === symlink

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @basename    = '.'
    @filesize    = 0 # Is 0 initially.
    @mtime       = Time.parse('2014-10-25 00:00:00 +0200') # <- This requires time.
    @raw         = DEFAULT_INPUT_FOR_THE_ENTRY_CALLED_RAW
    @server_type = 'Unix'
    @symlink     = false
    @directory   = true
    @file        = false
    @device      = false
  end

  # ========================================================================= #
  # === filesize?
  # ========================================================================= #
  def filesize?
    @filesize
  end; alias filesize filesize? # === filesize

  # ========================================================================= #
  # === set_filesize
  # ========================================================================= #
  def set_filesize(i)
    @filesize = i.to_i
  end

  # ========================================================================= #
  # === show_the_line
  # ========================================================================= #
  def show_the_line
    e @raw
  end

  # ========================================================================= #
  # === is_directory
  #
  # Setter method to determine that we have found a directory.
  # ========================================================================= #
  def is_directory
    @symlink   = false
    @file      = false
    @device    = false
    @directory = true
  end

  # ========================================================================= #
  # === line?
  # ========================================================================= #
  def line?
    @raw
  end

  # ========================================================================= #
  # === name
  # ========================================================================= #
  def name
    @basename
  end; alias return_name name # === return_name
       alias name?       name # === name?
       alias filename?   name # === filename?

  # ========================================================================= #
  # === determine_timestamp
  # ========================================================================= #
  def determine_timestamp
    if @splitted
      new_string = "#{@splitted[5]}-#{@splitted[6]}-#{@splitted[7]}"
      @mtime = new_string
    else
      @mtime = nil
    end
  end

  # ========================================================================= #
  # === parsed_timestamp?
  # ========================================================================= #
  def parsed_timestamp?
    Time.parse(@mtime)
  end

  # ========================================================================= #
  # === determine_filesize
  #
  # This method will determine the filesize of our remote entry at
  # hand. The input may look like this:
  #   "-rw-rw-rw-   1 web      netscape    10548 May 30  2011 Chained.rb"
  # So when we split on ' ', the filesize will be entry number 4.
  # ========================================================================= #
  def determine_filesize
    if @splitted
      set_filesize(@splitted[4])
    else
      set_filesize(0)
    end
  end

  # ========================================================================= #
  # === determine_basename
  # ========================================================================= #
  def determine_basename
    if @splitted
      @basename = @splitted.last
    else
      @basename = raw
    end
  end

  # ========================================================================= #
  # === is_a_directory?
  # ========================================================================= #
  def is_a_directory?
    @basename.end_with?('/') or @raw.start_with?('d')
  end; alias is_directory? is_a_directory? # === is_directory?

  # ========================================================================= #
  # === file_or_directory?
  #
  # This will return the string 'file' or the string 'directory',
  # depending on whether we have a file or a directory.
  # ========================================================================= #
  def file_or_directory?
    if is_directory?
      'directory'
    else
      'file'
    end
  end; alias file_or_directory file_or_directory? # === file_or_directory
       alias type?             file_or_directory? # === type?

  # ========================================================================= #
  # === directory?
  # ========================================================================= #
  def directory?
    @directory
  end; alias dir  directory? # === dir
       alias dir? directory? # === dir?

  # ========================================================================= #
  # === is_a_file?
  # ========================================================================= #
  def is_a_file?
    !is_a_directory?
  end

  # ========================================================================= #
  # === mtime?
  # ========================================================================= #
  def mtime?
    @mtime
  end; alias mtime mtime? # === mtime

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file
  end; alias file file? # === file

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    case @raw[0,1]
    when 'd' # If it starts with a 'd', then it is a directory.
      is_directory
    end
    determine_basename
    determine_timestamp
    determine_filesize
  end

  # ========================================================================= #
  # === Entry.parse
  # ========================================================================= #
  def self.parse(i)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  array = ["-rw-r--r--    1 0          2                   0 Oct 29 19:23 DO NOT UPLOAD FILES HERE",
 "htdocs/",
 "-rw-r--r--    1 22918231   22918231            9 Oct 29 19:32 test.md",
 "drwxr-xr-x    2 22918231   22918231         4096 Oct 30 10:59 test2"]
  array.map! {|line|
    FtpParadise::Entry.new(line)
  }
  pp array
  entry = array[1]
  e
pp entry
  e
  puts entry.file_or_directory?
  puts entry.type?
end