#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/directory_handling.rb'
# =========================================================================== #
module FtpParadise

class InteractiveFtp

  # ========================================================================= #
  # === ftp_remote_directory?
  # ========================================================================= #
  def ftp_remote_directory?
    ftp?.remote_directory?
  end; alias return_remote_pwd ftp_remote_directory? # === return_remote_pwd

  # ========================================================================= #
  # === report_remote_directory
  #
  # This method will report the remote directory.
  # ========================================================================= #
  def report_remote_directory(
      make_newline = true
    )
    if is_connected?
      remote_dir = sdir(ftp_remote_directory?)
      if make_newline
        e 'The remote directory is: '
        e "  #{sdir(remote_dir)}"
      else
        e 'The remote directory is: '+sdir(remote_dir)
      end
    else
      e 'No connection is available. You may have to re-connect again.'
    end
  end; alias show_remote_directory report_remote_directory # === show_remote_directory

  # ========================================================================= #
  # === remote_directory?
  # ========================================================================= #
  def remote_directory?
    @remote_directory
  end

  # ========================================================================= #
  # === set_remote_directory
  # ========================================================================= #
  def set_remote_directory(i)
    i = File.dirname(i) if ftp?.is_file?(i) # Get the base directory here.
    @remote_directory = i
    @array_history_remote_directories << @remote_directory
  end

  # ========================================================================= #
  # === upload_directory
  # ========================================================================= #
  def upload_directory(i)
    ftp?.upload_directory(i)
  end

  # ========================================================================= #
  # === remote_change_directory
  #
  # This method can be used to change directory on the remote host.
  #
  # An argument such as :show_remote_file_listing can be passed as second
  # argument to this method, in which case we will list the content of
  # the remote directory after the cd-action.
  #
  # If readline is used, as determined by the instance variable @use_readline,
  # then we will attempt to cd-<TAB>.
  #
  # Invocation example:
  #
  #   rcd htdocs
  #
  # ========================================================================= #
  def remote_change_directory(
      i                    = f?,
      optional_instruction = nil,
      be_verbose           = false
    )
    i = i.first if i.is_a? Array
    i = '..' if i.nil? # <- Tiny "safeguard".
    # ======================================================================= #
    # We must first check whether we input only numbers,
    # and whether such a directory does NOT exist. 
    # ======================================================================= #
    if (i =~ /^\d+$/) and !is_dir?(i)
      ftp?.update_file_listing
      i = ftp?.array_file_listing[i.to_i - 1]
    end
    if be_verbose
      e "Trying to cd into the remote directory `#{sfancy(i)}`."
    end
    case i # case tag
    when '?' # intercept ?
      e 'You went into these remote directories so far:'
      @array_history_remote_directories.each { |entry|
        e "- #{entry}"
      }
    else # This is the default.
      if i
        i.squeeze!('/') if i.include? '//' # Get rid of multiple //.
        e "Changing remote directory to `#{sdir(i)}` now."
      end
      begin
        ftp?.chdir(i)
        set_remote_directory(i)
        if optional_instruction
          case optional_instruction
          when :show_remote_file_listing
            show_remote_files
          end
        end
      rescue Exception => error
        pp error
      end
    end
  end; alias remote_cd remote_change_directory # === remote_cd

  # ========================================================================= #
  # === change_local_and_remote_directories
  #
  # We combine local cd and remote cd here.
  # ========================================================================= #
  def change_local_and_remote_directories(
      target = DEFAULT_LOCAL_DIRECTORY
    )
    change_directory(target, :be_silent) # Mandatory since as of 01.05.2011
    if are_we_connected?
      remote_change_directory(target) # Since as of August 2011.
    end
  end

  # ========================================================================= #
  # === set_start_directory=
  # ========================================================================= #
  def set_start_directory=(i) # set_start_directory=(i)
    change_directory(i)
  end

  # ========================================================================= #
  # === change_directory                                             (cd tag)
  #
  # This only changes for local directories, not remote directories.
  # ========================================================================= #
  def change_directory(
      i          = DEFAULT_LOCAL_DIRECTORY,
      be_verbose = true
    )
    i = '..' if i.nil?
    be_verbose = false if be_verbose == :be_silent
    i = i.to_s
    i.gsub!(/; pwd/,'') if i.include? '; pwd'
    begin
      i = get_env_of(i) if i.include? '$'
      if File.exist?(i)
        # =================================================================== #
        # Next, do the actual "change directory" action.
        # =================================================================== #
        Dir.chdir(i) # This will cd for local directories.
        i = return_pwd if i == '..'
        i = rds(return_pwd) unless i.include? '/'
        set_local_directory(i) # Set the new pwd here.
        report_local_directory if be_verbose
      else # else, the directory does not exist.
        e swarn('But the directory '+sdir(i)+' does not exist '\
          'locally in '+sdir(return_pwd)+'.')
      end
    rescue Exception => error
      e 'An exception occurred in '+swarn('change_directory()')
      e "The directory `#{sdir(i)}` does not exist."
      pp error # Should remove this and error perhaps.
    end
  end; alias cd change_directory # === cd

  # ========================================================================= #
  # === report_local_directory
  #
  # Use only this method when reporting the local directory.
  # ========================================================================= #
  def report_local_directory
    e 'The local directory is: '+N+
       @pad+sdir(return_pwd)
  end; alias report_local_dir report_local_directory # === report_local_dir

  # ========================================================================= #
  # === remote_create_directory
  # ========================================================================= #
  def remote_create_directory(
      i          = a?,
      be_verbose = true
    )
    if i.is_a? Array
      i.each {|entry| remote_create_directory(entry, be_verbose) }
    else
      # ===================================================================== #
      # Delegate towards the respective module-method next.
      # ===================================================================== #
      if be_verbose
        e "Creating the directory #{sdir(i)} next:"
      end
      FtpParadise.remote_create_directory(i, @ftp)
    end
  end

  # ========================================================================= #
  # === return_pwd
  # ========================================================================= #
  def return_pwd
    ("#{Dir.pwd}/").squeeze '/'
  end

end; end