#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === FtpParadise::InteractiveFtp
#
# Use this class here for interactive usage of the FTP paradise project.
#
# This class provides a commandline variant of the FtpParadise project,
# which allows you to interactively issue commands and instructions
# to a remote FTP server.
#
# No GUI code is otherwise stored herein.
# =========================================================================== #
# The official source for Ruby-FTP can be found at:
#
#   http://www.ruby-doc.org/stdlib/libdoc/net/ftp/rdoc/Net/FTP.html
#
# Usage example for this class here:
#
#   FtpParadise::InteractiveFtp.new
#   FtpParadise.interactive
#
# =========================================================================== #
# require 'ftp_paradise/interactive_ftp/interactive_ftp.rb'
# =========================================================================== #
require 'ftp_paradise/requires/common_basic_requires.rb'
require 'ftp_paradise/base/prototype.rb'
require 'ftp_paradise/toplevel_methods/data.rb'
require 'ftp_paradise/toplevel_methods/ftp_object.rb'
require 'ftp_paradise/toplevel_methods/login_name.rb'
require 'ftp_paradise/toplevel_methods/password.rb'
require 'ftp_paradise/toplevel_methods/remote_url.rb'
# =========================================================================== #
# Next ftp_connection/ files:
# =========================================================================== #
require 'ftp_paradise/connection/connection.rb'
require 'ftp_paradise/base/prototype.rb'

module FtpParadise

class InteractiveFtp # === FtpParadise::InteractiveFtp

  include ::FtpParadise::Base

  # ========================================================================= #
  # Next, load up interactive_ftp-specific .rb files:
  # ========================================================================= #
  require 'ftp_paradise/interactive_ftp/constants.rb'
  require 'ftp_paradise/interactive_ftp/help.rb'
  require 'ftp_paradise/interactive_ftp/initialize.rb'
  require 'ftp_paradise/interactive_ftp/main_loop.rb'
  require 'ftp_paradise/interactive_ftp/menu.rb'
  require 'ftp_paradise/interactive_ftp/misc.rb'
  require 'ftp_paradise/interactive_ftp/mode.rb'
  require 'ftp_paradise/interactive_ftp/remove.rb'
  require 'ftp_paradise/interactive_ftp/reset.rb'
  require 'ftp_paradise/interactive_ftp/run.rb'
  require 'ftp_paradise/interactive_ftp/show.rb'
  require 'ftp_paradise/interactive_ftp/upload.rb'
  require 'ftp_paradise/interactive_ftp/user_input.rb'

  begin
    require 'rcfiles'
  rescue LoadError; end

  # ========================================================================= #
  # === verbose_truth
  # ========================================================================= #
  def verbose_truth(i = '')
    VerboseTruth[i]
  end

  # ========================================================================= #
  # === set_url
  # ========================================================================= #
  def set_url(i)
    ::FtpParadise.set_remote_host(i)
  end

  # ========================================================================= #
  # === ftp?
  # ========================================================================= #
  def ftp?
    FtpParadise.ftp_object?
  end; alias ftp_object? ftp? # === ftp_object?

  # ========================================================================= #
  # === connected_to?
  #
  # Feedback where we are connected to.
  # ========================================================================= #
  def connected_to?
    if ftp?.open?
      report_where_we_are_connected_to
      report_remote_directory # Added Dec 2011.
      # report_local_dir
      # unless ftp?.empty?
      #   e 'The remote file listing is:'
      #   show_remote_listing
      # end # Disabled the above chunk as of Apr 2017.
    else
      e 'We are not connected to any remote FTP server currently.'
    end
  end

  # ========================================================================= #
  # === initialize_ftp_object                                     (reset tag)
  #
  # The FTP Subsection is gathered here. This method is called from
  # the method reset().
  # ========================================================================= #
  def initialize_ftp_object
    FtpParadise.use_this_as_ftp_object(
      FtpParadise::Connection.new(:default, false)
    )
    ftp?.enable_debug if debug? # Set the @debug variable inside of @ftp to true.
    set_passive
    use_ascii_mode
  end

  # ========================================================================= #
  # === f?
  # ========================================================================= #
  def f?
    @first_argument # First argument. Keep in mind that the second argument is stored in @s.
  end; alias f f? # === f

  # ========================================================================= #
  # === connect_to_this_remote_url
  # ========================================================================= #
  def connect_to_this_remote_url(
      i          = remote_url?,
      be_verbose = :be_verbose
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    if be_verbose
      e "Connecting to this remote URL: #{sfancy(i)}" 
    end
    FtpParadise.use_this_as_ftp_object(
      FtpParadise::Connection.new(i)
    )
    ftp?.login(
      FtpParadise.login_name?,
      password?
    )
  end

  # ========================================================================= #
  # === rename
  #
  # This will attempt a remote rename-action.
  # ========================================================================= #
  def rename(a, b)
    begin
      a = convert_number_to_file(a)
      ftp?.rename(a, b)
    rescue Net::FTPPermError
      e "The file at #{sfile(i)} does not seem to exist - thus, "\
        "we can not rename it."
    end
  end

  # ========================================================================= #
  # === upload_this_file
  # ========================================================================= #
  def upload_this_file(
      i = f?
     )
    if i.is_a? Array
      i.each {|entry| upload_this_file(entry) }
    else
      e 'Trying to upload this file `'+sfile(i)+'` next:'
      ftp?.puttextfile(i)
    end
  end

  # ========================================================================= #
  # === reset_instance_variables_that_hold_all_arguments
  #
  # Use this method to initialite the instance variables that will hold
  # the arguments to methods, from the user input.
  #
  # @s refers to the second argument.
  # ========================================================================= #    
  def reset_instance_variables_that_hold_all_arguments
    @splitted = @s = @first_argument = @all_arguments = nil
  end

  # ========================================================================= #
  # === set_file
  # ========================================================================= #
  def set_file(i = nil) # Use only this to set @file variable.
    if i # Safeguard against nil.
      i = i.first if i.is_a? Array
      i = i.to_s
      i = Dir.pwd+'/'+i if ! i.include?('/') # Since Dec 2011 we try to use only the absolute path.
      unless File.exist?(i)
        i << '.c' unless i.include? '.' # Append .c to files, unless the file exists.
      end
      i = FtpParadise.rds(i)
    end
    @file = i # This is the only allowed way to modify @file.
  end

  # ========================================================================= #
  # === be_verbose=
  # ========================================================================= #
  def be_verbose=(i)
    @be_verbose = i
  end

  # ========================================================================= #
  # === exit_program                                             (exit tag)
  # ========================================================================= #
  def exit_program # Use only this method to exit i-ftp.
    e 'Bye from '+return_title+'! (@run_mode was '+simp(@run_mode.to_s)+')'
    if @run_mode == :standalone # Not embedded.
      @run_mode = :exit
      disconnect
      exit
    else
      @run_mode = :break
    end
  end

  # ========================================================================= #
  # === download_remote_file                                 (download tag)
  #
  # This method will download a remote file. When we download a remote
  # file, we will also set the remote base directory since Dec 2011.
  # ========================================================================= #
  def download_remote_file(
      file = DEFAULT_REMOTE_FILE
    )
    file = DEFAULT_REMOTE_FILE if file.nil?
    if file.include? '*'
      file = ftp?.find_matches_for(file)
    end
    if file.is_a? Array
      file.each {|the_file| download_remote_file(the_file) }
    else
      if file =~ /^\d+$/ # if is a number. DEBUG HERE THOUGH.
        ftp?.update
        file = ftp?.array_file_listing[file.to_i - 1][0]
      end
      set_remote_directory(file)
      unless file == '*'
        opne 'Trying to download `'+sfile(file)+'` now.'
        # return_value = ftp?.download(file)
        result = ftp?.download(file) # Delegate to the parent class.
        if result
          @array_downloaded_files << file
          set_file(File.basename(file)) # setting new again.
          if @open_in_default_editor # Works on @file now, as we did set it before.
            open_this_file_in_editor(@file)
          end
        else
          opne 'An exception occurred - you probably lack permissions to'
          opne 'download that remote file.'
        end
      end
    end
  end

  # ========================================================================= #
  # === has_unwanted_file_suffix?
  #
  # The following method will return true or false.
  #
  # I needed it so that the editor does not open every file that was
  # thrown at it.
  # ========================================================================= #
  def has_unwanted_file_suffix?(i)
    return_value = false
    case File.extname(i).delete('.')
    when 'xz'
      return_value = true
    end
    return return_value
  end

  # ========================================================================= #
  # === debug_it
  # ========================================================================= #
  def debug_it # debug tag
    ftp?.debug(true)
    # pp ftp?.array_file_listing
  end

  # ========================================================================= #
  # === set_local_directory
  # ========================================================================= #
  def set_local_directory(
      i = Dir.pwd
    )
    i = i.to_s.dup
    i << '/' unless i.end_with? '/'
    i = FtpParadise.rds(i)
    @local_directory = i
  end

  # ========================================================================= #
  # === show_colours
  # ========================================================================= #
  def show_colours # Use this to show the colours.
    pp Colours.colours?
  end

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @debug
  end

  # ========================================================================= #
  # === save_config
  # ========================================================================= #
  def save_config
    @config.save
  end

  # ========================================================================= #
  # === stat_remote_file                                         (stat tag)
  #
  # This method can be called via '?33' for instance, or via 'stat 33'.
  # ========================================================================= #
  def stat_remote_file(i)
    file = ftp?[i] # Obtain that file.
    matches = ftp?.array_file_listing.select {|key,value| key == file }
    matches.each { |entry|
      e '  '+entry[0]+': '+sfancy(entry[2]+' KB')
    }
  end

  # ========================================================================= #
  # === show_local_entries
  # ========================================================================= #
  def show_local_entries
    show_local_files(false)
  end

  # ========================================================================= #
  # === return_local_files
  #
  # Return local files. Directories will not be included here.
  # ========================================================================= #
  def return_local_files
    entries = Dir['*']
    entries = Dir[Dir.pwd+'/*'] if show_full_names? # Overrule in this case.
    entries.reject! {|entry| File.directory? entry }
    return entries
  end; alias file_listing return_local_files # === file_listing

  # ========================================================================= #
  # === return_local_entries
  #
  # Returns all local entries.
  # ========================================================================= #
  def return_local_entries
    entries = Dir['*']
    entries = Dir[Dir.pwd+'/*'] if show_full_names? # Overrule in this case.
    return entries
  end

  # ========================================================================= #
  # === coloured_remote_host?
  # ========================================================================= #
  def coloured_remote_host?
    sfancy(remote_host?)
  end

  # ========================================================================= #
  # === remote_host?
  # ========================================================================= #
  def remote_host?
    ftp?.host?.to_s
  end

  # ========================================================================= #
  # === enable_anonymous_login
  #
  # We use this method to enable anonymous logins.
  # ========================================================================= #
  def enable_anonymous_login
    opne 'We will enable anonymous login now.'
    set_password :anonymous
    set_user_name 'anonymous'
  end

  # ========================================================================= #
  # === create_directory
  # ========================================================================= #
  def create_directory(i, be_verbose = false)
    be_verbose = true if be_verbose == :be_verbose
    e 'Now creating local directory '+sdir(i)+'.' if be_verbose
    FileUtils.mkdir_p(i)
  end

  # ========================================================================= #
  # === report_file
  #
  # Here we simply report which file we would upload.
  # ========================================================================= #
  def report_file
    if file?
      e 'The file that we would upload is: '
      e '  '+sfile(file?)
    else
      e 'No file was specified yet.'
    end
  end

  # ========================================================================= #
  # === move_local_file
  # ========================================================================= #
  def move_local_file(from, to)
    e 'Next moving (local) file from '+sfile(from)+' to '+sfile(to)+'.'
    if File.exist? from
      FileUtils.mv(from, to)
    else
      e 'No (local) file called '+sfile(from)+' could be found.' 
    end
  end

  # ========================================================================= #
  # === disconnect
  #
  # Disconnect the remote connection by invoking the quit() method on
  # the FTP object.
  # ========================================================================= #
  def disconnect(be_verbose = true)
    if ftp?.did_we_connect?
      e 'Disconnected from `'+coloured_remote_host?+'`.' if be_verbose
    end
    ftp?.quit
    exit
  end

  # ========================================================================= #
  # === move_remote_file
  #
  # We expect an Array as input.
  #
  # To test this method, do:
  #   rmv DONE_Prfungsfragen_Hartig_Einheiten_2_3_5_9_10_11_12_15_16_21.pdf PDFs/
  #   mmv vimrc vim
  # ========================================================================= #
  def move_remote_file(i)
    from = i[0]
    to   = i[1]
    if is_directory?(to)
      to << '/' unless to.end_with? '/'
      to << File.basename(from)
    end
    e 'Next moving from '+sfancy(from)+' to '+sfancy(to)+'.'
    ftp?.move_file(from, to)
  end

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint
    Signal.trap('SIGINT') {
      e 'Please use "q" to quit.'
    }
  end

  # ========================================================================= #
  # === disable
  # ========================================================================= #
  def disable(i = nil)
    i = i.to_s.to_sym
    case i
    when :base_directory, :full_names
      do_not_show_full_names
    when :debug
      @config.debug = false
      ftp?.disable_debug
      save_config
    end
    e 'Disabling '+sfancy(i.to_s)+' now.' unless i.to_s.empty?
  end

  # ========================================================================= #
  # === toggle_show_full_names
  # ========================================================================= #
  def toggle_show_full_names
    @config.show_full_names = ! @config.show_full_names
  end

  # ========================================================================= #
  # === do_not_show_full_names
  # ========================================================================= #
  def do_not_show_full_names
    ftp?.show_full_names = false
    @config.show_full_names = false # Sync.
  end

  # ========================================================================= #
  # === do_show_full_names
  # ========================================================================= #
  def do_show_full_names
    ftp?.show_full_names = true
    @config.show_full_names = true # Sync.
  end

  # ========================================================================= #
  # === will_we_show_full_names
  # ========================================================================= #
  def will_we_show_full_names
    e 'Will we now show full names? '+
      verbose_truth(show_full_names?)
  end

  # ========================================================================= #
  # === report_last_response
  # ========================================================================= #
  def report_last_response
    e 'The last response was: '+simp(ftp?.ftp.last_response.to_s)
  end

  # ========================================================================= #
  # === report_last_response_code
  # ========================================================================= #
  def report_last_response_code
    e 'The last response code was: '+simp(ftp?.ftp.last_response_code.to_s)
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i)
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === check_commandline_arguments                         (commandline tag)
  #
  # This is called from within method enter_loop().
  # ========================================================================= #
  def check_commandline_arguments(
      i = @commandline_arguments
    )
    if i.is_a?(String) and i.include?(',') # Here we assume a chained-instruction.
      i = i.split(',')
    end
    i = [i].flatten.compact
    i.each {|entry|
      case entry
      when '',nil # Do nothing in this case.
      when '--disable-colours','--disable-colors','nocolours','nocol',
           /^-?-?nocol$/
        e 'Now disabling colours.'
        FtpParadise.disable_colours
      when *ARRAY_HELP_OPTIONS
        show_commandline_help
        exit
      when 'BINARY'
        set_binary :be_verbose
      else
        entry = entry.to_s
        if File.exist?(entry)
          set_file(entry)
          @commandline_arguments.reject! {|line|
            line.include? entry
          }
        end
      end
    }
  end

  # ========================================================================= #
  # === show_commandline_help
  #
  # This method is different from "normal" help. The "normal" help will
  # be displayed when we are already running in interactive mode. The
  # commandline help however had will be run when we are invoking it
  # from the commandline.
  # ========================================================================= #
  def show_commandline_help
    opne 'These options are currently supported:'
    opn(use_colours?); Colours.ecomment '  --disable-colours # Run without colours.'
  end

  # ========================================================================= #
  # === show_welcome_message
  #
  # The initial startup message to welcome the user.
  # ========================================================================= #
  def show_welcome_message
    e 'Welcome to the '+return_title+' shell.'
    e 'Input '+sfancy('"help"')+' to show the help-section.' 
    e 'Please input commands next:'
  end

  # ========================================================================= #
  # === show_ftp_server_welcome
  #
  # Simply show the FTP server's welcome message.
  # ========================================================================= #
  def show_ftp_server_welcome
    e ftp?.welcome
  end

  # ========================================================================= #
  # === return_title
  # ========================================================================= #
  def return_title
    sfancy(TITLE)
  end

  # ========================================================================= #
  # === try_to_append_this_to_the_history
  #
  # Right now only 'last' is a forbidden entry.
  # ========================================================================= #
  def try_to_append_this_to_the_history(i)
    unless i.empty?
      @history << i unless ARRAY_FORBIDDEN_HISTORY_COMMANDS.include? i
    end
  end

  # ========================================================================= #
  # === load_config
  #
  # This method will load the configuration for this class.
  # ========================================================================= #
  def load_config
    @config = ::Roebe::Configuration::Configuration.new(FtpParadise.yaml_directory?)
    @open_in_default_editor = OPEN_IN_DEFAULT_EDITOR
    if @config.has_entry? 'open_in_default_editor'
      @open_in_default_editor = @config.open_in_default_editor
    end
    if @config.respond_to? :debug
      @debug = @config.debug # Obtain (or rather, overrule) the @debug variable.
    end
  end

  # ========================================================================= #
  # === report_port
  # ========================================================================= #
  def report_port
    e 'The port in use is: '+sfancy(ftp?.port)
  end

  # ========================================================================= #
  # === get_env_of
  # ========================================================================= #
  def get_env_of(i)
    ConvertGlobalEnv.convert(i, ConvertGlobalEnv.report_errors?)
  end

  # ========================================================================= #
  # === file?
  #
  # This may always be a String.
  # ========================================================================= #
  def file?
    @file.to_s
  end

  # ========================================================================= #
  # === consider_reporting_file
  # ========================================================================= #
  def consider_reporting_file
    if file?.empty?
      e 'No file has been assigned yet.'
      e 'You can set a specific file via '+simportant('"set_file"')+'.'
    else
      report_file
    end
  end

  # ========================================================================= #
  # === toggle                                                 (toggle tag)
  #
  # Simply toggle the value for whether we @config.show_full_names or not.
  # ========================================================================= #
  def toggle(
      i = :names
    )
    i = i.to_s
    i = 'mode' if i.empty? # Safeguard.
    case i
    # ======================================================================= #
    # === mode
    # ======================================================================= #
    when 'mode'
      if @mode == :binary
        use_ascii_mode
      else
        use_binary_mode
      end
    when 'names','name'
      toggle_show_full_names
      will_we_show_full_names
    end
    e 'We will now toggle '+sfancy(i)+' to '+simp(@mode.to_s)+'.'
  end; alias show_full_names toggle # === show_full_names

  # ========================================================================= #
  # === show_user_and_password
  # ========================================================================= #
  def show_user_and_password
    ftp?.show_user_and_password
  end

  # ========================================================================= #
  # === show_host_user_name_port_and_password
  # ========================================================================= #
  def show_host_user_name_port_and_password
    ftp?.show_host_user_name_port_and_password
  end

  # ========================================================================= #
  # === show_local_files
  #
  # This method shows the local file listing. That is, all the entries in
  # the given directory.
  # ========================================================================= #
  def show_local_files(show_only_directories = true)
    _ = Dir.pwd
    if Dir.entries(_).size == 2 # Dir.empty?, we assume that we have '.' and '..' only
      e 'The directory `'+sdir(_)+'` is empty.'
    else
      e 'Showing local files now from `'+sdir(_)+'`:'
      e ('=' * 55)+' '+FtpParadise.yellow('LOCAL FILES')+' '+('=' * 12)
      entries = return_local_entries
      entries.reject! {|x| ! File.directory?(x) } if show_only_directories
      entries.sort.each_with_index { |item, index|
        case File.ftype(item)
        when 'directory'
          item = sdir(item+'/')
        when 'file'
          item = sfile(item)
        end
        e '('+('%2s' % (index+1).to_s)+') '+FtpParadise.rds(item)
      }
      cliner
    end
  end

  # ========================================================================= #
  # === convert_number_to_file
  #
  # Use this to convert a number.
  #   convert_number_to_file(i) if i =~ /^\d+$/
  # ========================================================================= #
  def convert_number_to_file(number)
    number = number.to_s.chomp
    if number =~ /^\d+$/ # if is a number
      ftp?.update_file_listing
      number = ftp?.array_file_listing[number.to_i - 1][0]
    end
    return number
  end

  # ========================================================================= #
  # === is_dir?
  #
  # Return true or false, depending on whether the input is a directory
  # or whether it is not.
  # ========================================================================= #
  def is_dir?(i)
    ftp?.is_directory?(i)
  end; alias is_directory? is_dir? # === is_directory?

  # ========================================================================= #
  # === is_file?
  # ========================================================================= #
  def is_file?(i)
    ftp?.is_file?(i)
  end

  # ========================================================================= #
  # === this_remote_entry_exists?
  # ========================================================================= #
  def this_remote_entry_exists?(i)
    does_it_exist = false
    does_it_exist = true if is_dir?  i # This will also check whether it exists.
    does_it_exist = true if is_file? i
    return does_it_exist
  end

  # ========================================================================= #
  # === show_date_sorted_entries
  #
  # This method will show the sorted entries of the given remote directory
  # at hand.
  # ========================================================================= #
  def show_date_sorted_entries
    dataset = ftp?.array_file_listing?
    reverse_sort_dateset = dataset.sort_by {|_a,_b_,_c, timestamp|
      timestamp
    }.reverse
    ftp?.show_remote_listing(reverse_sort_dateset)
  end

  # ========================================================================= #
  # === set_name
  # ========================================================================= #
  def set_name(i)
    i = i.to_s
    e 'Setting user-name to `'+sfancy(i)+'` now.'
    ftp?.set_user_name(i)
  end; alias set_user_name set_name # === set_user_name

  # ========================================================================= #
  # === show_full_names?
  # ========================================================================= #
  def show_full_names?
    @config.show_full_names
  end

  # ========================================================================= #
  # === create_remote_directory
  # ========================================================================= #
  def create_remote_directory(i)
    ftp?.mkdir(i)
  end

  # ========================================================================= #
  # === opne
  # ========================================================================= #
  def opne(message)
    opnn; e message
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn(i = {namespace: NAMESPACE, use_colours: use_colours?})
    Opn.opn(i)
  end

  # ========================================================================= #
  # === set_login
  # ========================================================================= #
  def set_login(i)
    ftp?.set_login_name(i)
  end

  # ========================================================================= #
  # === user_input?
  # ========================================================================= #
  def user_input?
    @user_input
  end

  # ========================================================================= #
  # === report_where_we_are_connected_to
  # ========================================================================= #
  def report_where_we_are_connected_to
    if are_we_connected?
      e 'We are connected to: '+simportant(ftp?.remote?)
    else
      e 'We are not connected. Invoke "do_connect" to connect to a FTP host.'
    end
  end

  # ========================================================================= #
  # === return_remote_files
  #
  # We call into FtpParadise::Connection here.
  # ========================================================================= #
  def return_remote_files(mode = nil)
    ftp?.show_full_names if show_full_names? # Display the full path here.
    ftp?.return_remote_files
  end

  # ========================================================================= #
  # === are_we_connected?
  # ========================================================================= #
  def are_we_connected?
    @are_we_connected
  end; alias is_connected? are_we_connected? # === is_connected?

  # ========================================================================= #
  # === automatically_connect_on_startup_of_the_interactive_ftp_shell?
  # ========================================================================= #
  def automatically_connect_on_startup_of_the_interactive_ftp_shell?
    _ = FILE_AUTOMATICALLY_CONNECT_ON_STARTUP_OF_THE_INTERACTIVE_FTP_SHELL
    if File.exist? _
      YAML.load_file(_)
    else
      false # In this case we default to false.
    end
  end

  # ========================================================================= #
  # === startup_actions
  #
  # This method can be used to designate startup actions - which are actions
  # that are run when the interactive ftp-shell is first started.
  #
  # Presently (Sep 2018) we will only consider two startup actions:
  #
  #   (1) Consider making use of the Readline Module
  #   (2) auto-connecting on startup if a certain yaml-configuration file
  #       has an entry set to true, and if we are on roebe.
  #
  # ========================================================================= #
  def startup_actions
    # ======================================================================= #
    # We will next try to make use of the Readline-module, if it is available:
    # ======================================================================= #
    try_to_use_readline
    if are_we_on_roebe? and automatically_connect_on_startup_of_the_interactive_ftp_shell?
      # ===================================================================== #
      # The next is for my home system mostly; not sure if it is worth
      # to make it more general for other people too.
      # ===================================================================== #
      do_login_to_shevy_and_consider_showing_the_remote_files
    end
  end

  # ========================================================================= #
  # === set_active
  # ========================================================================= #
  def set_active
    ftp?.set_active
  end

  # ========================================================================= #
  # === datafile?
  # ========================================================================= #
  def datafile?
    '$MY_FTP/YAML/roebe_ftp.yml'
  end

  # ========================================================================= #
  # === display_prompt
  # ========================================================================= #
  def display_prompt
    print PROMPT_TO_USE
  end

  # ========================================================================= #
  # === download?
  # ========================================================================= #
  def download?
    e 'The remote file we would download is: '
    e
    e "  #{sfile(DEFAULT_REMOTE_FILE)}"
    e
  end

  # ========================================================================= #
  # === do_login_to_shevy_and_consider_showing_the_remote_files
  # ========================================================================= #
  def do_login_to_shevy_and_consider_showing_the_remote_files
    do_login(:shevy)
    if SHOW_REMOTE_FILES_UPON_SUCCESSFUL_LOGIN
      show_remote_files :also_show_filesize # Also display this since as of March 2015.
    end
  end

  # ========================================================================= #
  # === do_login
  #
  # If the first argument is :dont_login_yet, which is the default,
  # we will not yet connect.
  # ========================================================================= #
  def do_login(
      i = :dont_login_yet
    )
    i = nil if i == :dont_login_yet
    if i
      # ===================================================================== #
      # Next, feed this information to the ftp? object.
      # ===================================================================== #
      ftp?.set_data(i)
      ftp?.do_login # bl $RUBY_FTP/ftp_library.rb
      @are_we_connected = true
    end
  end

end

# =========================================================================== #
# === FtpParadise.interactive
#
# Convenience method to instantiate a new class Interactive instance.
# =========================================================================== #
def self.interactive(i = ARGV)
  FtpParadise::InteractiveFtp.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  FtpParadise.interactive(ARGV)
end # iftp