#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'ftp_paradise/toplevel_methods/ftp_object.rb'
# =========================================================================== #
module FtpParadise

  require 'net/ftp'
  require 'ftp_paradise/toplevel_methods/e.rb'
  require 'ftp_paradise/toplevel_methods/login_name.rb'
  require 'ftp_paradise/toplevel_methods/password.rb'
  require 'ftp_paradise/toplevel_methods/remote_url.rb'

  # ========================================================================= #
  # === @ftp_object
  #
  # This is the main Net FTP object that we will use to connect to
  # some remote host. It will be stored on the module-level so that
  # we can also use module-level methods such as
  # FtpParadise.ftp_object? for easy access ways.
  # ========================================================================= #
  @ftp_object = nil

  require 'yaml'
  require 'ftp_paradise/constants/roebe.rb'
  # ========================================================================= #
  # === FtpParadise.initialize_ftp_object
  #
  # Use this method if you wish to initialize a new FTP object. This will
  # be, by default, a blank-state initializer.
  # ========================================================================= #
  def self.initialize_ftp_object(use_this_url = nil)
    case use_this_url
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      use_this_url = YAML.load_file(FILE_ROEBE_FTP)['podserver']['url']
    end
    @ftp_object = Net::FTP.new(use_this_url)
  end

  # ========================================================================= #
  # === FtpParadise.do_login                                      (login tag)
  #
  # This method combines setting the remote URL, before doing the login
  # action.
  # ========================================================================= #
  def self.do_login(
      use_this_as_the_remote_url = nil,
      user                       = :default,
      passwd                     = :default_or_nil,
      acct                       = nil
    )
    if use_this_as_the_remote_url and FtpParadise.ftp_object?.nil?
      FtpParadise.set_remote_url(use_this_as_the_remote_url)
    end
    case use_this_as_the_remote_url
    # ======================================================================= #
    # === :bytehost
    # ======================================================================= #
    when :bytehost
      FtpParadise.set_remote_url(:infer)
      use_this_as_the_remote_url = remote_url?
    end
    # ======================================================================= #
    # Sanitize the default user (user name aka login name).
    # ======================================================================= #
    case user
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      if is_on_roebe?
        set_login_name(:infer)
        user = login_name?
      else
        user = 'anonymous'
      end
    end
    # ======================================================================= #
    # Sanitize the given password next.
    # ======================================================================= #
    case passwd
    # ======================================================================= #
    # === :default_or_nil
    # ======================================================================= #
    when :default_or_nil
      if is_on_roebe?
        set_password(:infer)
        passwd = password?
      else
        passwd = nil
      end
    end
    # ======================================================================= #
    # puts "Debug: user name is: #{user} password is: #{passwd} account is: #{acct}"
    # ======================================================================= #
    @ftp_object.login(
      user, passwd, acct
    ) # Delegate to the main object here.
    return @ftp_object # And mandatory return here.
  end

  # ========================================================================= #
  # === FtpParadise.use_this_as_ftp_object
  #
  # Assignment to a new FTP object can come through this method.
  # ========================================================================= #
  def self.use_this_as_ftp_object(i)
    @ftp_object = i
  end; self.instance_eval { alias assign_ftp_object= use_this_as_ftp_object } # === FtpParadise.assign_ftp_object=
       self.instance_eval { alias ftp_object=        use_this_as_ftp_object } # === FtpParadise.ftp_object=

  # ========================================================================= #
  # === FtpParadise.list
  # ========================================================================= #
  def self.list(i = 'n*')
    @ftp_object.list(i)
  end; self.instance_eval { alias ll list } # === FtpParadise.ll

  # ========================================================================= #
  # === FtpParadise.close
  # ========================================================================= #
  def self.close
    @ftp_object.close
  end

  # ========================================================================= #
  # === FtpParadise.mlsd
  #
  # This method will return an Array of the entries of the directory
  # specified by pathname.
  #
  # Each entry has the facts (e.g., size, last modification time, etc.)
  # and the pathname. If a block is given, it iterates through the listing.
  #
  # If pathname is omitted, the current directory is assumed.
  # ========================================================================= #
  def self.mlsd(
      pathname = nil
    )
    @ftp_object.mlsd(pathname)
  end

  # ========================================================================= #
  # === FtpParadise.delete
  #
  # Use this to delete a remote file.
  # ========================================================================= #
  def self.delete(i)
    FtpParadise.ftp_object.delete(i)
  end; self.instance_eval { alias remove_this_file delete } # === FtpParadise.remove_this_file

  # ========================================================================= #
  # === FtpParadise.getbinaryfile
  # ========================================================================= #
  def self.getbinaryfile(
      a = 'nif.rb-0.91.gz',
      b = 'nif.gz',
      c = 1024
    )
    @ftp_object.getbinaryfile(a,b,c)
  end

  # ========================================================================= #
  # === FtpParadise.remote_host?
  # ========================================================================= #
  def self.remote_host?
    if @ftp_object
      @ftp_object.remote_host?
    else
      nil
    end
  end; self.instance_eval { alias host? remote_host? } # === FtpParadise.host?

  # ========================================================================= #
  # === FtpParadise.chdir                                            (cd tag)
  #
  # Change directory through this method here.
  #
  # Usage Example:
  #
  #   FtpParadise.chdir('pub/lang/ruby/contrib')
  #   FtpParadise.cd('pub/lang/ruby/contrib')
  #
  # ========================================================================= #
  def self.chdir(i)
    @ftp_object.chdir(i)
  end; self.instance_eval { alias cd        chdir } # === FtpParadise.cd
       self.instance_eval { alias remote_cd chdir } # === FtpParadise.remote_cd

  # ========================================================================= #
  # === FtpParadise.ftp_object
  #
  # Reader method to return the @ftp_object.
  #
  # This method should ideally come last in this .rb file.
  # ========================================================================= #
  def self.ftp_object
    @ftp_object
  end; self.instance_eval { alias ftp_object? ftp_object } # === FtpParadise.ftp_object?
       self.instance_eval { alias object      ftp_object } # === FtpParadise.object

  # ========================================================================= #
  # === FtpParadise.show_list
  #
  # By default this method will exclude the '.' and '..' entries.
  # ========================================================================= #
  def self.show_list
    result = @ftp_object.list
    result.each {|line|
      e line unless line.end_with? '.'
    }
  end

  # ========================================================================= #
  # === FtpParadise.remote_mkdir
  #
  # This method will create a remote directory. Make sure to have
  # a valid (active) FTP connection before you invoke this method.
  # ========================================================================= #
  def self.remote_mkdir(
      create_this_remote_directory,
      connection           = @ftp_object,
      connect_to_this_host = :shevy
    )
    if create_this_remote_directory.is_a? Array
      create_this_remote_directory.each {|name_of_the_remote_directory|
        remote_mkdir(
          name_of_the_remote_directory,
          connection,
          connect_to_this_host
        )
      }
    else
      create_this_remote_directory = create_this_remote_directory.to_s

      if connection.nil?
        require 'ftp_paradise/connection/connection.rb'
        connection = FtpParadise::Connection.new(connect_to_this_host) { :do_not_run_yet }
        connection.be_silent
        connection.do_login
      end

      begin
        connection.mkdir(create_this_remote_directory)
      rescue Net::FTPPermError => error
        pp error
        e "No directory called #{create_this_remote_directory}/ could be created"
        e 'as such a (remote) directory already exists.'
      end

    end
  end; self.instance_eval { alias create_directory        remote_mkdir } # === FtpParadise.create_directory
       self.instance_eval { alias remote_create_directory remote_mkdir } # === FtpParadise.remote_create_directory

end

if __FILE__ == $PROGRAM_NAME
  FtpParadise.initialize_ftp_object(:default)
  result = FtpParadise.do_login :bytehost
  pp result.list
  pp result
  FtpParadise.cd 'htdocs'
  FtpParadise.show_list
  FtpParadise.remote_mkdir('test')
  FtpParadise.show_list
end # ftpobject