#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === FtpWebApp
#
# This file requires these projects:
#
#   sinatra
#   html_tags
#   colours
#   ftp_paradise
#
# =========================================================================== #
require 'sinatra'
require 'html_tags'
require 'colours/autoinclude'
require 'ftp_paradise'
require 'cliner'

# =========================================================================== #
# We must remove method :body first before continuing.
# =========================================================================== #
HtmlTags.class_eval { remove_method(:body) }

class FtpWebApp < Sinatra::Base

  include HtmlTags

  set :port, USE_THIS_PORT
  set :logging, true

  USE_THIS_PORT = 81
  WHERE_TO = :SHEVY

  # ========================================================================= #
  # Initialize the Ftp-Connection here.
  # ========================================================================= #
  FTP = FtpParadise::Connection.new(:dont_run)

  # ========================================================================= #
  # The various constants for the URL actions.
  # ========================================================================= #
  ROOT       = '/'
  ABOUT      = '/about/?'
  CONNECT    = '/connect/?'
  CONNECT_TO = '/connect_to/?'
  STATUS     = '/status/?'
  LISTING    = '/listing/?'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    super
    run if run_already
  end

  # ========================================================================= #
  # === link_to_everything                          (everything tag, all tag)
  # ========================================================================= #
  def link_to_everything
    div(
      :id => 'intralink',
      :css_style => 'margin: 0.55em;
      margin-top:1.05em;
      margin-left:0.45em;
      margin-right:0.45em;
      padding-top: 2px;
      padding:5px;
      border:1px dotted black;'
    ) {
      h2('Available actions for the www-ftp part:','','',
      'margin-top:1px; margin-bottom:1px;padding:0.2em')+
      link_to(:connect,'margin-left:1em')+
      link_to(:status)+
      link_to(:root)+
      link_to(:listing)
    }
  end

  # ========================================================================= #
  # === link_to
  # ========================================================================= #
  def link_to(i, optional_css = '')
    case i.to_s
    when 'root'
      i = '/'
    end
    a(i.to_s, :make_newline,'','padding-left:1.2em;'+optional_css)
  end

  # ========================================================================= #
  # === do_connect
  # ========================================================================= #
  def do_connect(
      where_to = WHERE_TO
    )
    FTP.connect :to => where_to
  end

  # ========================================================================= #
  # localhost:81/halt
  # ========================================================================= #
  get '/halt' do 
    halt 403, 'Halting everything.'
  end

  # ========================================================================= #
  # === /about
  #
  # localhost:81/about
  # About tag
  # ========================================================================= #
  get ABOUT do
    _  = h2 'This is the Web Interface for the FtpParadise project.'
    _ << 'If you wish to connect to a remote site, try:'+br
    _ << h4('  connect SERVER_URL_HERE','','',
         'color:slateblue; margin-left:1em')
    return _
  end

  # ========================================================================= #
  # /listing                                                      (listing tag)
  # ========================================================================= #
  get LISTING do
    do_connect
    _ = h2('Remote File listing (of <b>'+WHERE_TO.to_s+'</b>):','','',
           'text-decoration: underline')
    FTP.array_remote_files.
      reject {|entry| entry[0,1] == '.'}.
      each_with_index {|entry, index|
      _ << (index+1).to_s+' <b>'+link(
        'http://'+FTP.connected_to?+'/'+entry.to_s
      )+'</b>'+br+"\n"
    }
    _+link_to_everything
  end

  # ========================================================================= #
  # === status tag
  #
  # localhost:81/status
  # ========================================================================= #
  get STATUS do
    do_connect
    show_status
  end

  # ========================================================================= #
  # localhost:81/status
  # ========================================================================= #
  get CONNECT do
    h2('Please give us an argument to use.','red')+
    h3('If you are unsure what kind of argument, simply append the login '+
       'host you wish to connect to, such as /SHEVY.')+br+
       'Example: /connect_to/SHEVY'+
       a('connect/SHEVY', :make_newline)
  end

  # ========================================================================= #
  # === show_status
  # ========================================================================= #
  def show_status
    _ = h2 'The status is as follows:'+br
    _ << 'We are connected to: '+b(FTP.connected_to?,
          '','','text-decoration: underline')+br+br
    _ << FTP.status?+br
    _ << link_to_everything
  end

  # ========================================================================= #
  # localhost:81/connect
  # ========================================================================= #
  [
    CONNECT+'/:connect',
    CONNECT_TO+'/:connect'
  ].each {|entry|
    get(entry) do
      FTP.connect :to => params[:connect]
      _ = 'Now connecting to ...'+params[:connect].to_s+
          show_status+br+link_to(:root)
      return _
    end
  }

  # ========================================================================= #
  # The root at / is shown here.
  # localhost:81/
  # ========================================================================= #
  get ROOT do
    h1('Welcome to the web-interface of the FtpParadise project. ',
       '','','margin-top:0.5em')+
    h2('We will now show which actions are possible:','','',
      'margin-bottom:1px; margin-left:1em')+
      link_to_everything
  end

  # ========================================================================= #
  # === report_port
  # ========================================================================= #
  def report_port
    cliner {
      e 'We will use the port '+sfancy(port?)+' ( http://localhost:'+port?+'/ ).'
    }
  end

  # ========================================================================= #
  # === port?
  # ========================================================================= #
  def port?
    USE_THIS_PORT.to_s
  end

  # ========================================================================= #
  # === not_found
  #
  # In case the action was not found, continue here.
  #
  # localhost:81/not_existing
  # ========================================================================= #
  not_found do
    status 404
    h1('The action '+request.path_info+' was not found.')+
    link_to_everything
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_port
    self.class.run!
  end

end

if __FILE__ == $PROGRAM_NAME
   FtpWebApp.new
end # rb sinatra_web_interface.rb